/**
 * Any schema type.
 */
type AnySchema<TOutput = any> = BaseSchema<any, TOutput> & {
    schema: 'any';
};
/**
 * Creates a any schema.
 *
 * @param pipe A validation and transformation pipe.
 *
 * @returns A any schema.
 */
declare function any(pipe?: Pipe<any>): AnySchema;

/**
 * Any schema type.
 */
type AnySchemaAsync<TOutput = any> = BaseSchemaAsync<any, TOutput> & {
    schema: 'any';
};
/**
 * Creates an async any schema.
 *
 * @param pipe A validation and transformation pipe.
 *
 * @returns An async any schema.
 */
declare function anyAsync(pipe?: PipeAsync<any>): AnySchemaAsync;

/**
 * Array schema type.
 */
type ArraySchema<TArrayItem extends BaseSchema, TOutput = Output<TArrayItem>[]> = BaseSchema<Input<TArrayItem>[], TOutput> & {
    schema: 'array';
    array: {
        item: TArrayItem;
    };
};
/**
 * Creates a array schema.
 *
 * @param item The item schema.
 * @param pipe A validation and transformation pipe.
 *
 * @returns A array schema.
 */
declare function array<TArrayItem extends BaseSchema>(item: TArrayItem, pipe?: Pipe<Output<TArrayItem>[]>): ArraySchema<TArrayItem>;
/**
 * Creates a array schema.
 *
 * @param item The item schema.
 * @param error The error message.
 * @param pipe A validation and transformation pipe.
 *
 * @returns A array schema.
 */
declare function array<TArrayItem extends BaseSchema>(item: TArrayItem, error?: ErrorMessage, pipe?: Pipe<Output<TArrayItem>[]>): ArraySchema<TArrayItem>;

/**
 * Array schema async type.
 */
type ArraySchemaAsync<TArrayItem extends BaseSchema | BaseSchemaAsync, TOutput = Output<TArrayItem>[]> = BaseSchemaAsync<Input<TArrayItem>[], TOutput> & {
    schema: 'array';
    array: {
        item: TArrayItem;
    };
};
/**
 * Creates an async array schema.
 *
 * @param item The item schema.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An async array schema.
 */
declare function arrayAsync<TArrayItem extends BaseSchema | BaseSchemaAsync>(item: TArrayItem, pipe?: PipeAsync<Output<TArrayItem>[]>): ArraySchemaAsync<TArrayItem>;
/**
 * Creates an async array schema.
 *
 * @param item The item schema.
 * @param error The error message.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An async array schema.
 */
declare function arrayAsync<TArrayItem extends BaseSchema | BaseSchemaAsync>(item: TArrayItem, error?: ErrorMessage, pipe?: PipeAsync<Output<TArrayItem>[]>): ArraySchemaAsync<TArrayItem>;

/**
 * Array path item type.
 */
type ArrayPathItem = {
    schema: 'array';
    input: any[];
    key: number;
    value: any;
};

/**
 * Bigint schema type.
 */
type BigintSchema<TOutput = bigint> = BaseSchema<bigint, TOutput> & {
    schema: 'bigint';
};
/**
 * Creates a bigint schema.
 *
 * @param pipe A validation and transformation pipe.
 *
 * @returns A bigint schema.
 */
declare function bigint(pipe?: Pipe<bigint>): BigintSchema;
/**
 * Creates a bigint schema.
 *
 * @param error The error message.
 * @param pipe A validation and transformation pipe.
 *
 * @returns A bigint schema.
 */
declare function bigint(error?: ErrorMessage, pipe?: Pipe<bigint>): BigintSchema;

/**
 * Bigint schema async type.
 */
type BigintSchemaAsync<TOutput = bigint> = BaseSchemaAsync<bigint, TOutput> & {
    schema: 'bigint';
};
/**
 * Creates an async bigint schema.
 *
 * @param pipe A validation and transformation pipe.
 *
 * @returns An async bigint schema.
 */
declare function bigintAsync(pipe?: PipeAsync<bigint>): BigintSchemaAsync;
/**
 * Creates an async bigint schema.
 *
 * @param error The error message.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An async bigint schema.
 */
declare function bigintAsync(error?: ErrorMessage, pipe?: PipeAsync<bigint>): BigintSchemaAsync;

/**
 * Blob schema type.
 */
type BlobSchema<TOutput = Blob> = BaseSchema<Blob, TOutput> & {
    schema: 'blob';
};
/**
 * Creates a blob schema.
 *
 * @param pipe A validation and transformation pipe.
 *
 * @returns A blob schema.
 */
declare function blob(pipe?: Pipe<Blob>): BlobSchema;
/**
 * Creates a blob schema.
 *
 * @param error The error message.
 * @param pipe A validation and transformation pipe.
 *
 * @returns A blob schema.
 */
declare function blob(error?: ErrorMessage, pipe?: Pipe<Blob>): BlobSchema;

/**
 * Blob schema async type.
 */
type BlobSchemaAsync<TOutput = Blob> = BaseSchemaAsync<Blob, TOutput> & {
    schema: 'blob';
};
/**
 * Creates an async blob schema.
 *
 * @param pipe A validation and transformation pipe.
 *
 * @returns An async blob schema.
 */
declare function blobAsync(pipe?: PipeAsync<Blob>): BlobSchemaAsync;
/**
 * Creates an async blob schema.
 *
 * @param error The error message.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An async blob schema.
 */
declare function blobAsync(error?: ErrorMessage, pipe?: PipeAsync<Blob>): BlobSchemaAsync;

/**
 * Boolean schema type.
 */
type BooleanSchema<TOutput = boolean> = BaseSchema<boolean, TOutput> & {
    schema: 'boolean';
};
/**
 * Creates a boolean schema.
 *
 * @param pipe A validation and transformation pipe.
 *
 * @returns A boolean schema.
 */
declare function boolean(pipe?: Pipe<boolean>): BooleanSchema;
/**
 * Creates a boolean schema.
 *
 * @param error The error message.
 * @param pipe A validation and transformation pipe.
 *
 * @returns A boolean schema.
 */
declare function boolean(error?: ErrorMessage, pipe?: Pipe<boolean>): BooleanSchema;

/**
 * Boolean schema async type.
 */
type BooleanSchemaAsync<TOutput = boolean> = BaseSchemaAsync<boolean, TOutput> & {
    schema: 'boolean';
};
/**
 * Creates an async boolean schema.
 *
 * @param pipe A validation and transformation pipe.
 *
 * @returns An async boolean schema.
 */
declare function booleanAsync(pipe?: PipeAsync<boolean>): BooleanSchemaAsync;
/**
 * Creates an async boolean schema.
 *
 * @param error The error message.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An async boolean schema.
 */
declare function booleanAsync(error?: ErrorMessage, pipe?: PipeAsync<boolean>): BooleanSchemaAsync;

/**
 * Date schema type.
 */
type DateSchema<TOutput = Date> = BaseSchema<Date, TOutput> & {
    schema: 'date';
};
/**
 * Creates a date schema.
 *
 * @param pipe A validation and transformation pipe.
 *
 * @returns A date schema.
 */
declare function date(pipe?: Pipe<Date>): DateSchema;
/**
 * Creates a date schema.
 *
 * @param error The error message.
 * @param pipe A validation and transformation pipe.
 *
 * @returns A date schema.
 */
declare function date(error?: ErrorMessage, pipe?: Pipe<Date>): DateSchema;

/**
 * Date schema async type.
 */
type DateSchemaAsync<TOutput = Date> = BaseSchemaAsync<Date, TOutput> & {
    schema: 'date';
};
/**
 * Creates an async date schema.
 *
 * @param pipe A validation and transformation pipe.
 *
 * @returns An async date schema.
 */
declare function dateAsync(pipe?: PipeAsync<Date>): DateSchemaAsync;
/**
 * Creates an async date schema.
 *
 * @param error The error message.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An async date schema.
 */
declare function dateAsync(error?: ErrorMessage, pipe?: PipeAsync<Date>): DateSchemaAsync;

/**
 * Enum type.
 */
type Enum<TOption extends string = string> = Readonly<[TOption, ...TOption[]]> | [TOption, ...TOption[]];

/**
 * Enum schema type.
 */
type EnumSchema<TEnum extends Enum, TOutput = TEnum[number]> = BaseSchema<TEnum[number], TOutput> & {
    schema: 'enum';
    enum: TEnum;
};
/**
 * Creates a enum schema.
 *
 * @param enumValue The enum value.
 * @param error The error message.
 *
 * @returns A enum schema.
 */
declare function enumType<TOption extends string, TEnum extends Enum<TOption>>(enumValue: TEnum, error?: ErrorMessage): EnumSchema<TEnum>;

/**
 * Enum schema async type.
 */
type EnumSchemaAsync<TEnum extends Enum, TOutput = TEnum[number]> = BaseSchemaAsync<TEnum[number], TOutput> & {
    schema: 'enum';
    enum: TEnum;
};
/**
 * Creates an async enum schema.
 *
 * @param enumValue The enum value.
 * @param error The error message.
 *
 * @returns An async enum schema.
 */
declare function enumTypeAsync<TOption extends string, TEnum extends Enum<TOption>>(enumValue: TEnum, error?: ErrorMessage): EnumSchemaAsync<TEnum>;

/**
 * Class enum type.
 */
type Class = abstract new (...args: any) => any;
/**
 * Instance schema type.
 */
type InstanceSchema<TClass extends Class, TOutput = InstanceType<TClass>> = BaseSchema<InstanceType<TClass>, TOutput> & {
    schema: 'instance';
    class: TClass;
};
/**
 * Creates an instance schema.
 *
 * @param of The class of the instance.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An instance schema.
 */
declare function instance<TClass extends Class>(of: TClass, pipe?: Pipe<InstanceType<TClass>>): InstanceSchema<TClass>;
/**
 * Creates an instance schema.
 *
 * @param of The class of the instance.
 * @param error The error message.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An instance schema.
 */
declare function instance<TClass extends Class>(of: TClass, error?: ErrorMessage, pipe?: Pipe<InstanceType<TClass>>): InstanceSchema<TClass>;

/**
 * Instance schema type.
 */
type InstanceSchemaAsync<TClass extends Class, TOutput = InstanceType<TClass>> = BaseSchemaAsync<InstanceType<TClass>, TOutput> & {
    schema: 'instance';
    class: TClass;
};
/**
 * Creates an async instance schema.
 *
 * @param of The class of the instance.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An async instance schema.
 */
declare function instanceAsync<TClass extends Class>(of: TClass, pipe?: PipeAsync<InstanceType<TClass>>): InstanceSchemaAsync<TClass>;
/**
 * Creates an async instance schema.
 *
 * @param of The class of the instance.
 * @param error The error message.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An async instance schema.
 */
declare function instanceAsync<TClass extends Class>(of: TClass, error?: ErrorMessage, pipe?: PipeAsync<InstanceType<TClass>>): InstanceSchemaAsync<TClass>;

/**
 * Literal type.
 */
type Literal = number | string | boolean | symbol | bigint;

/**
 * Literal schema type.
 */
type LiteralSchema<TLiteral extends Literal, TOutput = TLiteral> = BaseSchema<TLiteral, TOutput> & {
    schema: 'literal';
    literal: TLiteral;
};
/**
 * Creates a literal schema.
 *
 * @param literal The literal value.
 * @param error The error message.
 *
 * @returns A literal schema.
 */
declare function literal<TLiteral extends Literal>(literal: TLiteral, error?: ErrorMessage): LiteralSchema<TLiteral>;

/**
 * Literal schema async type.
 */
type LiteralSchemaAsync<TLiteral extends Literal, TOutput = TLiteral> = BaseSchemaAsync<TLiteral, TOutput> & {
    schema: 'literal';
    literal: TLiteral;
};
/**
 * Creates an async literal schema.
 *
 * @param literal The literal value.
 * @param error The error message.
 *
 * @returns An async literal schema.
 */
declare function literalAsync<TLiteral extends Literal>(literal: TLiteral, error?: ErrorMessage): LiteralSchemaAsync<TLiteral>;

/**
 * Map path item type.
 */
type MapPathItem = {
    schema: 'map';
    input: Map<any, any>;
    key: any;
    value: any;
};
/**
 * Map input inference type.
 */
type MapInput<TMapKey extends BaseSchema | BaseSchemaAsync, TMapValue extends BaseSchema | BaseSchemaAsync> = Map<Input<TMapKey>, Input<TMapValue>>;
/**
 * Map output inference type.
 */
type MapOutput<TMapKey extends BaseSchema | BaseSchemaAsync, TMapValue extends BaseSchema | BaseSchemaAsync> = Map<Output<TMapKey>, Output<TMapValue>>;

/**
 * Map schema type.
 */
type MapSchema<TMapKey extends BaseSchema, TMapValue extends BaseSchema, TOutput = MapOutput<TMapKey, TMapValue>> = BaseSchema<MapInput<TMapKey, TMapValue>, TOutput> & {
    schema: 'map';
    map: {
        key: TMapKey;
        value: TMapValue;
    };
};
/**
 * Creates a map schema.
 *
 * @param key The key schema.
 * @param value The value schema.
 * @param pipe A validation and transformation pipe.
 *
 * @returns A map schema.
 */
declare function map<TMapKey extends BaseSchema, TMapValue extends BaseSchema>(key: TMapKey, value: TMapValue, pipe?: Pipe<MapOutput<TMapKey, TMapValue>>): MapSchema<TMapKey, TMapValue>;
/**
 * Creates a map schema.
 *
 * @param key The key schema.
 * @param value The value schema.
 * @param error The error message.
 * @param pipe A validation and transformation pipe.
 *
 * @returns A map schema.
 */
declare function map<TMapKey extends BaseSchema, TMapValue extends BaseSchema>(key: TMapKey, value: TMapValue, error?: ErrorMessage, pipe?: Pipe<MapOutput<TMapKey, TMapValue>>): MapSchema<TMapKey, TMapValue>;

/**
 * Map schema async type.
 */
type MapSchemaAsync<TMapKey extends BaseSchema | BaseSchemaAsync, TMapValue extends BaseSchema | BaseSchemaAsync, TOutput = MapOutput<TMapKey, TMapValue>> = BaseSchemaAsync<MapInput<TMapKey, TMapValue>, TOutput> & {
    schema: 'map';
    map: {
        key: TMapKey;
        value: TMapValue;
    };
};
/**
 * Creates an async map schema.
 *
 * @param key The key schema.
 * @param value The value schema.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An async map schema.
 */
declare function mapAsync<TMapKey extends BaseSchema | BaseSchemaAsync, TMapValue extends BaseSchema | BaseSchemaAsync>(key: TMapKey, value: TMapValue, pipe?: PipeAsync<MapOutput<TMapKey, TMapValue>>): MapSchemaAsync<TMapKey, TMapValue>;
/**
 * Creates an async map schema.
 *
 * @param key The key schema.
 * @param value The value schema.
 * @param error The error message.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An async map schema.
 */
declare function mapAsync<TMapKey extends BaseSchema | BaseSchemaAsync, TMapValue extends BaseSchema | BaseSchemaAsync>(key: TMapKey, value: TMapValue, error?: ErrorMessage, pipe?: PipeAsync<MapOutput<TMapKey, TMapValue>>): MapSchemaAsync<TMapKey, TMapValue>;

/**
 * NaN schema type.
 */
type NanSchema<TOutput = number> = BaseSchema<number, TOutput> & {
    schema: 'nan';
};
/**
 * Creates a NaN schema.
 *
 * @param error The error message.
 *
 * @returns A NaN schema.
 */
declare function nan(error?: ErrorMessage): NanSchema;

/**
 * NaN schema async type.
 */
type NanSchemaAsync<TOutput = number> = BaseSchemaAsync<number, TOutput> & {
    schema: 'nan';
};
/**
 * Creates an async NaN schema.
 *
 * @param error The error message.
 *
 * @returns An async NaN schema.
 */
declare function nanAsync(error?: ErrorMessage): NanSchemaAsync;

/**
 * Native enum type.
 */
type NativeEnum = {
    [key: string]: string | number;
    [key: number]: string;
};
/**
 * Native enum schema type.
 */
type NativeEnumSchema<TNativeEnum extends NativeEnum, TOutput = TNativeEnum[keyof TNativeEnum]> = BaseSchema<TNativeEnum[keyof TNativeEnum], TOutput> & {
    schema: 'native_enum';
    nativeEnum: TNativeEnum;
};
/**
 * Creates a enum schema.
 *
 * @param nativeEnum The native enum value.
 * @param error The error message.
 *
 * @returns A enum schema.
 */
declare function nativeEnum<TNativeEnum extends NativeEnum>(nativeEnum: TNativeEnum, error?: ErrorMessage): NativeEnumSchema<TNativeEnum>;

/**
 * Native enum schema async type.
 */
type NativeEnumSchemaAsync<TNativeEnum extends NativeEnum, TOutput = TNativeEnum[keyof TNativeEnum]> = BaseSchemaAsync<TNativeEnum[keyof TNativeEnum], TOutput> & {
    schema: 'native_enum';
    nativeEnum: TNativeEnum;
};
/**
 * Creates an async enum schema.
 *
 * @param nativeEnum The native enum value.
 * @param error The error message.
 *
 * @returns An async enum schema.
 */
declare function nativeEnumAsync<TNativeEnum extends NativeEnum>(nativeEnum: TNativeEnum, error?: ErrorMessage): NativeEnumSchemaAsync<TNativeEnum>;

/**
 * Never schema type.
 */
type NeverSchema = BaseSchema<never> & {
    schema: 'never';
};
/**
 * Creates a never schema.
 *
 * @param error The error message.
 *
 * @returns A never schema.
 */
declare function never(error?: ErrorMessage): NeverSchema;

/**
 * Never schema async type.
 */
type NeverSchemaAsync = BaseSchemaAsync<never> & {
    schema: 'never';
};
/**
 * Creates an async never schema.
 *
 * @param error The error message.
 *
 * @returns An async never schema.
 */
declare function neverAsync(error?: ErrorMessage): NeverSchemaAsync;

/**
 * Non nullable type.
 */
type NonNullable$1<T> = T extends null ? never : T;
/**
 * Non nullable schema type.
 */
type NonNullableSchema<TWrapped extends BaseSchema, TOutput = NonNullable$1<Output<TWrapped>>> = BaseSchema<NonNullable$1<Input<TWrapped>>, TOutput> & {
    schema: 'non_nullable';
    wrapped: TWrapped;
};
/**
 * Creates a non nullable schema.
 *
 * @param wrapped The wrapped schema.
 * @param error The error message.
 *
 * @returns A non nullable schema.
 */
declare function nonNullable<TWrapped extends BaseSchema>(wrapped: TWrapped, error?: ErrorMessage): NonNullableSchema<TWrapped>;

/**
 * Non nullable schema async type.
 */
type NonNullableSchemaAsync<TWrapped extends BaseSchema | BaseSchemaAsync, TOutput = NonNullable$1<Output<TWrapped>>> = BaseSchemaAsync<NonNullable$1<Input<TWrapped>>, TOutput> & {
    schema: 'non_nullable';
    wrapped: TWrapped;
};
/**
 * Creates an async non nullable schema.
 *
 * @param wrapped The wrapped schema.
 * @param error The error message.
 *
 * @returns An async non nullable schema.
 */
declare function nonNullableAsync<TWrapped extends BaseSchema | BaseSchemaAsync>(wrapped: TWrapped, error?: ErrorMessage): NonNullableSchemaAsync<TWrapped>;

/**
 * Non nullish type.
 */
type NonNullish<T> = T extends null | undefined ? never : T;
/**
 * Non nullish schema type.
 */
type NonNullishSchema<TWrapped extends BaseSchema, TOutput = NonNullish<Output<TWrapped>>> = BaseSchema<NonNullish<Input<TWrapped>>, TOutput> & {
    schema: 'non_nullish';
    wrapped: TWrapped;
};
/**
 * Creates a non nullish schema.
 *
 * @param wrapped The wrapped schema.
 * @param error The error message.
 *
 * @returns A non nullish schema.
 */
declare function nonNullish<TWrapped extends BaseSchema>(wrapped: TWrapped, error?: ErrorMessage): NonNullishSchema<TWrapped>;

/**
 * Non nullish schema async type.
 */
type NonNullishSchemaAsync<TWrapped extends BaseSchema | BaseSchemaAsync, TOutput = NonNullish<Output<TWrapped>>> = BaseSchemaAsync<NonNullish<Input<TWrapped>>, TOutput> & {
    schema: 'non_nullish';
    wrapped: TWrapped;
};
/**
 * Creates an async non nullish schema.
 *
 * @param wrapped The wrapped schema.
 * @param error The error message.
 *
 * @returns An async non nullish schema.
 */
declare function nonNullishAsync<TWrapped extends BaseSchema | BaseSchemaAsync>(wrapped: TWrapped, error?: ErrorMessage): NonNullishSchemaAsync<TWrapped>;

/**
 * Non optional type.
 */
type NonOptional<T> = T extends undefined ? never : T;
/**
 * Non optional schema type.
 */
type NonOptionalSchema<TWrapped extends BaseSchema, TOutput = NonOptional<Output<TWrapped>>> = BaseSchema<NonOptional<Input<TWrapped>>, TOutput> & {
    schema: 'non_optional';
    wrapped: TWrapped;
};
/**
 * Creates a non optional schema.
 *
 * @param wrapped The wrapped schema.
 * @param error The error message.
 *
 * @returns A non optional schema.
 */
declare function nonOptional<TWrapped extends BaseSchema>(wrapped: TWrapped, error?: ErrorMessage): NonOptionalSchema<TWrapped>;

/**
 * Non optional schema async type.
 */
type NonOptionalSchemaAsync<TWrapped extends BaseSchema | BaseSchemaAsync, TOutput = NonOptional<Output<TWrapped>>> = BaseSchemaAsync<NonOptional<Input<TWrapped>>, TOutput> & {
    schema: 'non_optional';
    wrapped: TWrapped;
};
/**
 * Creates an async non optional schema.
 *
 * @param wrapped The wrapped schema.
 * @param error The error message.
 *
 * @returns An async non optional schema.
 */
declare function nonOptionalAsync<TWrapped extends BaseSchema | BaseSchemaAsync>(wrapped: TWrapped, error?: ErrorMessage): NonOptionalSchemaAsync<TWrapped>;

/**
 * Nullable schema type.
 */
type NullableSchema<TWrapped extends BaseSchema, TDefault extends Input<TWrapped> | undefined = undefined, TOutput = TDefault extends undefined ? Output<TWrapped> | null : Output<TWrapped>> = BaseSchema<Input<TWrapped> | null, TOutput> & {
    schema: 'nullable';
    wrapped: TWrapped;
    get default(): TDefault;
};
/**
 * Creates a nullable schema.
 *
 * @param wrapped The wrapped schema.
 * @param default_ The default value.
 *
 * @returns A nullable schema.
 */
declare function nullable<TWrapped extends BaseSchema, TDefault extends Input<TWrapped> | undefined = undefined>(wrapped: TWrapped, default_?: TDefault | (() => TDefault)): NullableSchema<TWrapped, TDefault>;

/**
 * Nullable schema async type.
 */
type NullableSchemaAsync<TWrapped extends BaseSchema | BaseSchemaAsync, TDefault extends Input<TWrapped> | undefined | Promise<Input<TWrapped> | undefined> = undefined, TOutput = Awaited<TDefault> extends undefined ? Output<TWrapped> | null : Output<TWrapped>> = BaseSchemaAsync<Input<TWrapped> | null, TOutput> & {
    schema: 'nullable';
    wrapped: TWrapped;
    get default(): TDefault;
};
/**
 * Creates an async nullable schema.
 *
 * @param wrapped The wrapped schema.
 * @param default_ The default value.
 *
 * @returns An async nullable schema.
 */
declare function nullableAsync<TWrapped extends BaseSchema | BaseSchemaAsync, TDefault extends Input<TWrapped> | undefined | Promise<Input<TWrapped> | undefined> = undefined>(wrapped: TWrapped, default_?: TDefault | (() => TDefault)): NullableSchemaAsync<TWrapped, TDefault>;

/**
 * Nullish schema type.
 */
type NullishSchema<TWrapped extends BaseSchema, TDefault extends Input<TWrapped> | undefined = undefined, TOutput = TDefault extends undefined ? Output<TWrapped> | null | undefined : Output<TWrapped>> = BaseSchema<Input<TWrapped> | null | undefined, TOutput> & {
    schema: 'nullish';
    wrapped: TWrapped;
    get default(): TDefault;
};
/**
 * Creates a nullish schema.
 *
 * @param wrapped The wrapped schema.
 * @param default_ The default value.
 *
 * @returns A nullish schema.
 */
declare function nullish<TWrapped extends BaseSchema, TDefault extends Input<TWrapped> | undefined = undefined>(wrapped: TWrapped, default_?: TDefault | (() => TDefault)): NullishSchema<TWrapped, TDefault>;

/**
 * Nullish schema async type.
 */
type NullishSchemaAsync<TWrapped extends BaseSchema | BaseSchemaAsync, TDefault extends Input<TWrapped> | undefined | Promise<Input<TWrapped> | undefined> = undefined, TOutput = Awaited<TDefault> extends undefined ? Output<TWrapped> | null | undefined : Output<TWrapped>> = BaseSchemaAsync<Input<TWrapped> | null | undefined, TOutput> & {
    schema: 'nullish';
    wrapped: TWrapped;
    get default(): TDefault;
};
/**
 * Creates an async nullish schema.
 *
 * @param wrapped The wrapped schema.
 * @param default_ The default value.
 *
 * @returns An async nullish schema.
 */
declare function nullishAsync<TWrapped extends BaseSchema | BaseSchemaAsync, TDefault extends Input<TWrapped> | undefined | Promise<Input<TWrapped> | undefined> = undefined>(wrapped: TWrapped, default_?: TDefault | (() => TDefault)): NullishSchemaAsync<TWrapped, TDefault>;

/**
 * Null schema type.
 */
type NullSchema<TOutput = null> = BaseSchema<null, TOutput> & {
    schema: 'null';
};
/**
 * Creates a null schema.
 *
 * @param error The error message.
 *
 * @returns A null schema.
 */
declare function nullType(error?: ErrorMessage): NullSchema;

/**
 * Null schema async type.
 */
type NullSchemaAsync<TOutput = null> = BaseSchemaAsync<null, TOutput> & {
    schema: 'null';
};
/**
 * Creates an async null schema.
 *
 * @param error The error message.
 *
 * @returns An async null schema.
 */
declare function nullTypeAsync(error?: ErrorMessage): NullSchemaAsync;

/**
 * Number schema type.
 */
type NumberSchema<TOutput = number> = BaseSchema<number, TOutput> & {
    schema: 'number';
};
/**
 * Creates a number schema.
 *
 * @param pipe A validation and transformation pipe.
 *
 * @returns A number schema.
 */
declare function number(pipe?: Pipe<number>): NumberSchema;
/**
 * Creates a number schema.
 *
 * @param error The error message.
 * @param pipe A validation and transformation pipe.
 *
 * @returns A number schema.
 */
declare function number(error?: ErrorMessage, pipe?: Pipe<number>): NumberSchema;

/**
 * Number schema async type.
 */
type NumberSchemaAsync<TOutput = number> = BaseSchemaAsync<number, TOutput> & {
    schema: 'number';
};
/**
 * Creates an async number schema.
 *
 * @param pipe A validation and transformation pipe.
 *
 * @returns An async number schema.
 */
declare function numberAsync(pipe?: PipeAsync<number>): NumberSchemaAsync;
/**
 * Creates an async number schema.
 *
 * @param error The error message.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An async number schema.
 */
declare function numberAsync(error?: ErrorMessage, pipe?: PipeAsync<number>): NumberSchemaAsync;

/**
 * Object shape async type.
 */
type ObjectShapeAsync = Record<string, BaseSchema<any> | BaseSchemaAsync<any>>;
/**
 * Object schema async type.
 */
type ObjectSchemaAsync<TObjectShape extends ObjectShapeAsync, TOutput = ObjectOutput<TObjectShape>> = BaseSchemaAsync<ObjectInput<TObjectShape>, TOutput> & {
    schema: 'object';
    object: TObjectShape;
};
/**
 * Creates an async object schema.
 *
 * @param object The object schema.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An async object schema.
 */
declare function objectAsync<TObjectShape extends ObjectShapeAsync>(object: TObjectShape, pipe?: PipeAsync<ObjectOutput<TObjectShape>>): ObjectSchemaAsync<TObjectShape>;
/**
 * Creates an async object schema.
 *
 * @param object The object schema.
 * @param error The error message.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An async object schema.
 */
declare function objectAsync<TObjectShape extends ObjectShapeAsync>(object: TObjectShape, error?: ErrorMessage, pipe?: PipeAsync<ObjectOutput<TObjectShape>>): ObjectSchemaAsync<TObjectShape>;

/**
 * Object path item type.
 */
type ObjectPathItem = {
    schema: 'object';
    input: Record<string, any>;
    key: string;
    value: any;
};
/**
 * Required object keys type.
 */
type RequiredKeys<TObject extends object> = {
    [TKey in keyof TObject]: undefined extends TObject[TKey] ? never : TKey;
}[keyof TObject];
/**
 * Optional object keys type.
 */
type OptionalKeys<TObject extends object> = {
    [TKey in keyof TObject]: undefined extends TObject[TKey] ? TKey : never;
}[keyof TObject];
/**
 * Object with question marks type.
 */
type WithQuestionMarks<TObject extends object> = Pick<TObject, RequiredKeys<TObject>> & Partial<Pick<TObject, OptionalKeys<TObject>>>;
/**
 * Object input inference type.
 */
type ObjectInput<TObjectShape extends ObjectShape | ObjectShapeAsync> = ResolveObject<WithQuestionMarks<{
    [TKey in keyof TObjectShape]: Input<TObjectShape[TKey]>;
}>>;
/**
 * Object output inference type.
 */
type ObjectOutput<TObjectShape extends ObjectShape | ObjectShapeAsync> = ResolveObject<WithQuestionMarks<{
    [TKey in keyof TObjectShape]: Output<TObjectShape[TKey]>;
}>>;

/**
 * Object shape type.
 */
type ObjectShape = Record<string, BaseSchema<any>>;
/**
 * Object schema type.
 */
type ObjectSchema<TObjectShape extends ObjectShape, TOutput = ObjectOutput<TObjectShape>> = BaseSchema<ObjectInput<TObjectShape>, TOutput> & {
    schema: 'object';
    object: TObjectShape;
};
/**
 * Creates an object schema.
 *
 * @param object The object schema.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An object schema.
 */
declare function object<TObjectShape extends ObjectShape>(object: TObjectShape, pipe?: Pipe<ObjectOutput<TObjectShape>>): ObjectSchema<TObjectShape>;
/**
 * Creates an object schema.
 *
 * @param object The object schema.
 * @param error The error message.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An object schema.
 */
declare function object<TObjectShape extends ObjectShape>(object: TObjectShape, error?: ErrorMessage, pipe?: Pipe<ObjectOutput<TObjectShape>>): ObjectSchema<TObjectShape>;

/**
 * Optional schema type.
 */
type OptionalSchema<TWrapped extends BaseSchema, TDefault extends Input<TWrapped> | undefined = undefined, TOutput = TDefault extends undefined ? Output<TWrapped> | undefined : Output<TWrapped>> = BaseSchema<Input<TWrapped> | undefined, TOutput> & {
    schema: 'optional';
    wrapped: TWrapped;
    get default(): TDefault;
};
/**
 * Creates a optional schema.
 *
 * @param wrapped The wrapped schema.
 * @param default_ The default value.
 *
 * @returns A optional schema.
 */
declare function optional<TWrapped extends BaseSchema, TDefault extends Input<TWrapped> | undefined = undefined>(wrapped: TWrapped, default_?: TDefault | (() => TDefault)): OptionalSchema<TWrapped, TDefault>;

/**
 * Optional schema async type.
 */
type OptionalSchemaAsync<TWrapped extends BaseSchema | BaseSchemaAsync, TDefault extends Input<TWrapped> | undefined | Promise<Input<TWrapped> | undefined> = undefined, TOutput = Awaited<TDefault> extends undefined ? Output<TWrapped> | undefined : Output<TWrapped>> = BaseSchemaAsync<Input<TWrapped> | undefined, TOutput> & {
    schema: 'optional';
    wrapped: TWrapped;
    get default(): TDefault;
};
/**
 * Creates an async optional schema.
 *
 * @param wrapped The wrapped schema.
 * @param default_ The default value.
 *
 * @returns An async optional schema.
 */
declare function optionalAsync<TWrapped extends BaseSchema | BaseSchemaAsync, TDefault extends Input<TWrapped> | undefined | Promise<Input<TWrapped> | undefined> = undefined>(wrapped: TWrapped, default_?: TDefault | (() => TDefault)): OptionalSchemaAsync<TWrapped, TDefault>;

/**
 * String schema type.
 */
type StringSchema<TOutput = string> = BaseSchema<string, TOutput> & {
    schema: 'string';
};
/**
 * Creates a string schema.
 *
 * @param pipe A validation and transformation pipe.
 *
 * @returns A string schema.
 */
declare function string(pipe?: Pipe<string>): StringSchema;
/**
 * Creates a string schema.
 *
 * @param error The error message.
 * @param pipe A validation and transformation pipe.
 *
 * @returns A string schema.
 */
declare function string(error?: ErrorMessage, pipe?: Pipe<string>): StringSchema;

/**
 * String schema async type.
 */
type StringSchemaAsync<TOutput = string> = BaseSchemaAsync<string, TOutput> & {
    schema: 'string';
};
/**
 * Creates an async string schema.
 *
 * @param pipe A validation and transformation pipe.
 *
 * @returns An async string schema.
 */
declare function stringAsync(pipe?: PipeAsync<string>): StringSchemaAsync;
/**
 * Creates an async string schema.
 *
 * @param error The error message.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An async string schema.
 */
declare function stringAsync(error?: ErrorMessage, pipe?: PipeAsync<string>): StringSchemaAsync;

/**
 * Record key type.
 */
type RecordKeyAsync = EnumSchema<any, string | number | symbol> | EnumSchemaAsync<any, string | number | symbol> | NativeEnumSchema<any, string | number | symbol> | NativeEnumSchemaAsync<any, string | number | symbol> | StringSchema<string | number | symbol> | StringSchemaAsync<string | number | symbol>;
/**
 * Record schema async type.
 */
type RecordSchemaAsync<TRecordValue extends BaseSchema | BaseSchemaAsync, TRecordKey extends RecordKeyAsync = StringSchema, TOutput = RecordOutput<TRecordKey, TRecordValue>> = BaseSchemaAsync<RecordInput<TRecordKey, TRecordValue>, TOutput> & {
    schema: 'record';
    record: {
        key: TRecordKey;
        value: TRecordValue;
    };
};
/**
 * Creates an async record schema.
 *
 * @param value The value schema.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An async record schema.
 */
declare function recordAsync<TRecordValue extends BaseSchema | BaseSchemaAsync>(value: TRecordValue, pipe?: PipeAsync<RecordOutput<StringSchema, TRecordValue>>): RecordSchemaAsync<TRecordValue>;
/**
 * Creates an async record schema.
 *
 * @param value The value schema.
 * @param error The error message.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An async record schema.
 */
declare function recordAsync<TRecordValue extends BaseSchema | BaseSchemaAsync>(value: TRecordValue, error?: ErrorMessage, pipe?: PipeAsync<RecordOutput<StringSchema, TRecordValue>>): RecordSchemaAsync<TRecordValue>;
/**
 * Creates an async record schema.
 *
 * @param key The key schema.
 * @param value The value schema.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An async record schema.
 */
declare function recordAsync<TRecordKey extends RecordKeyAsync, TRecordValue extends BaseSchema | BaseSchemaAsync>(key: TRecordKey, value: TRecordValue, pipe?: PipeAsync<RecordOutput<TRecordKey, TRecordValue>>): RecordSchemaAsync<TRecordValue, TRecordKey>;
/**
 * Creates an async record schema.
 *
 * @param key The key schema.
 * @param value The value schema.
 * @param error The error message.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An async record schema.
 */
declare function recordAsync<TRecordKey extends RecordKeyAsync, TRecordValue extends BaseSchema | BaseSchemaAsync>(key: TRecordKey, value: TRecordValue, error?: ErrorMessage, pipe?: PipeAsync<RecordOutput<TRecordKey, TRecordValue>>): RecordSchemaAsync<TRecordValue, TRecordKey>;

/**
 * Record path item type.
 */
type RecordPathItem = {
    schema: 'record';
    input: Record<string | number | symbol, any>;
    key: string | number | symbol;
    value: any;
};
/**
 * Record input inference type.
 */
type RecordInput<TRecordKey extends RecordKey | RecordKeyAsync, TRecordValue extends BaseSchema | BaseSchemaAsync> = ResolveObject<Record<Input<TRecordKey>, Input<TRecordValue>>>;
/**
 * Record output inference type.
 */
type RecordOutput<TRecordKey extends RecordKey | RecordKeyAsync, TRecordValue extends BaseSchema | BaseSchemaAsync> = ResolveObject<Record<Output<TRecordKey>, Output<TRecordValue>>>;

/**
 * Record key type.
 */
type RecordKey = EnumSchema<any, string | number | symbol> | NativeEnumSchema<any, string | number | symbol> | StringSchema<string | number | symbol>;
/**
 * Record schema type.
 */
type RecordSchema<TRecordValue extends BaseSchema, TRecordKey extends RecordKey = StringSchema, TOutput = RecordOutput<TRecordKey, TRecordValue>> = BaseSchema<RecordInput<TRecordKey, TRecordValue>, TOutput> & {
    schema: 'record';
    record: {
        key: TRecordKey;
        value: TRecordValue;
    };
};
/**
 * Creates a record schema.
 *
 * @param value The value schema.
 * @param pipe A validation and transformation pipe.
 *
 * @returns A record schema.
 */
declare function record<TRecordValue extends BaseSchema>(value: TRecordValue, pipe?: Pipe<RecordOutput<StringSchema, TRecordValue>>): RecordSchema<TRecordValue>;
/**
 * Creates a record schema.
 *
 * @param value The value schema.
 * @param error The error message.
 * @param pipe A validation and transformation pipe.
 *
 * @returns A record schema.
 */
declare function record<TRecordValue extends BaseSchema>(value: TRecordValue, error?: ErrorMessage, pipe?: Pipe<RecordOutput<StringSchema, TRecordValue>>): RecordSchema<TRecordValue>;
/**
 * Creates a record schema.
 *
 * @param key The key schema.
 * @param value The value schema.
 * @param pipe A validation and transformation pipe.
 *
 * @returns A record schema.
 */
declare function record<TRecordKey extends RecordKey, TRecordValue extends BaseSchema>(key: TRecordKey, value: TRecordValue, pipe?: Pipe<RecordOutput<TRecordKey, TRecordValue>>): RecordSchema<TRecordValue, TRecordKey>;
/**
 * Creates a record schema.
 *
 * @param key The key schema.
 * @param value The value schema.
 * @param error The error message.
 * @param pipe A validation and transformation pipe.
 *
 * @returns A record schema.
 */
declare function record<TRecordKey extends RecordKey, TRecordValue extends BaseSchema>(key: TRecordKey, value: TRecordValue, error?: ErrorMessage, pipe?: Pipe<RecordOutput<TRecordKey, TRecordValue>>): RecordSchema<TRecordValue, TRecordKey>;

/**
 * Recursive schema type.
 */
type RecursiveSchema<TSchemaGetter extends () => BaseSchema, TOutput = Output<ReturnType<TSchemaGetter>>> = BaseSchema<Input<ReturnType<TSchemaGetter>>, TOutput> & {
    schema: 'recursive';
    getter: TSchemaGetter;
};
/**
 * Creates a recursive schema.
 *
 * @param getter The schema getter.
 *
 * @returns A recursive schema.
 */
declare function recursive<TSchemaGetter extends () => BaseSchema>(getter: TSchemaGetter): RecursiveSchema<TSchemaGetter>;

/**
 * Recursive schema async type.
 */
type RecursiveSchemaAsync<TSchemaGetter extends () => BaseSchema | BaseSchemaAsync, TOutput = Output<ReturnType<TSchemaGetter>>> = BaseSchemaAsync<Input<ReturnType<TSchemaGetter>>, TOutput> & {
    schema: 'recursive';
    getter: TSchemaGetter;
};
/**
 * Creates an async recursive schema.
 *
 * @param getter The schema getter.
 *
 * @returns An async recursive schema.
 */
declare function recursiveAsync<TSchemaGetter extends () => BaseSchema | BaseSchemaAsync>(getter: TSchemaGetter): RecursiveSchemaAsync<TSchemaGetter>;

/**
 * Set path item type.
 */
type SetPathItem = {
    schema: 'set';
    input: Set<any>;
    key: number;
    value: any;
};
/**
 * Set output inference type.
 */
type SetInput<TSetValue extends BaseSchema | BaseSchemaAsync> = Set<Input<TSetValue>>;
/**
 * Set output inference type.
 */
type SetOutput<TSetValue extends BaseSchema | BaseSchemaAsync> = Set<Output<TSetValue>>;

/**
 * Set schema type.
 */
type SetSchema<TSetValue extends BaseSchema, TOutput = SetOutput<TSetValue>> = BaseSchema<SetInput<TSetValue>, TOutput> & {
    schema: 'set';
    set: {
        value: TSetValue;
    };
};
/**
 * Creates a set schema.
 *
 * @param value The value schema.
 * @param pipe A validation and transformation pipe.
 *
 * @returns A set schema.
 */
declare function set<TSetValue extends BaseSchema>(value: TSetValue, pipe?: Pipe<SetOutput<TSetValue>>): SetSchema<TSetValue>;
/**
 * Creates a set schema.
 *
 * @param value The value schema.
 * @param error The error message.
 * @param pipe A validation and transformation pipe.
 *
 * @returns A set schema.
 */
declare function set<TSetValue extends BaseSchema>(value: TSetValue, error?: ErrorMessage, pipe?: Pipe<SetOutput<TSetValue>>): SetSchema<TSetValue>;

/**
 * Set schema async type.
 */
type SetSchemaAsync<TSetValue extends BaseSchema | BaseSchemaAsync, TOutput = SetOutput<TSetValue>> = BaseSchemaAsync<SetInput<TSetValue>, TOutput> & {
    schema: 'set';
    set: {
        value: TSetValue;
    };
};
/**
 * Creates an async set schema.
 *
 * @param value The value schema.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An async set schema.
 */
declare function setAsync<TSetValue extends BaseSchema | BaseSchemaAsync>(value: TSetValue, pipe?: PipeAsync<SetOutput<TSetValue>>): SetSchemaAsync<TSetValue>;
/**
 * Creates an async set schema.
 *
 * @param value The value schema.
 * @param error The error message.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An async set schema.
 */
declare function setAsync<TSetValue extends BaseSchema | BaseSchemaAsync>(value: TSetValue, error?: ErrorMessage, pipe?: PipeAsync<SetOutput<TSetValue>>): SetSchemaAsync<TSetValue>;

/**
 * Special schema type.
 */
type SpecialSchema<TInput, TOutput = TInput> = BaseSchema<TInput, TOutput> & {
    schema: 'special';
};
/**
 * Creates a special schema.
 *
 * @param check The type check function.
 * @param pipe A validation and transformation pipe.
 *
 * @returns A special schema.
 */
declare function special<TInput>(check: (input: unknown) => boolean, pipe?: Pipe<TInput>): SpecialSchema<TInput>;
/**
 * Creates a special schema.
 *
 * @param check The type check function.
 * @param error The error message.
 * @param pipe A validation and transformation pipe.
 *
 * @returns A special schema.
 */
declare function special<TInput>(check: (input: unknown) => boolean, error?: ErrorMessage, pipe?: Pipe<TInput>): SpecialSchema<TInput>;

/**
 * Special schema async type.
 */
type SpecialSchemaAsync<TInput, TOutput = TInput> = BaseSchemaAsync<TInput, TOutput> & {
    schema: 'special';
};
/**
 * Creates an async special schema.
 *
 * @param check The type check function.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An async special schema.
 */
declare function specialAsync<TInput>(check: (input: unknown) => boolean | Promise<boolean>, pipe?: PipeAsync<TInput>): SpecialSchemaAsync<TInput>;
/**
 * Creates a special schema.
 *
 * @param check The type check function.
 * @param error The error message.
 * @param pipe A validation and transformation pipe.
 *
 * @returns A special schema.
 */
declare function specialAsync<TInput>(check: (input: unknown) => boolean | Promise<boolean>, error?: ErrorMessage, pipe?: PipeAsync<TInput>): SpecialSchemaAsync<TInput>;

/**
 * Symbol schema type.
 */
type SymbolSchema<TOutput = symbol> = BaseSchema<symbol, TOutput> & {
    schema: 'symbol';
};
/**
 * Creates a symbol schema.
 *
 * @param error The error message.
 *
 * @returns A symbol schema.
 */
declare function symbol(error?: ErrorMessage): SymbolSchema;

/**
 * Symbol schema async type.
 */
type SymbolSchemaAsync<TOutput = symbol> = BaseSchemaAsync<symbol, TOutput> & {
    schema: 'symbol';
};
/**
 * Creates an async symbol schema.
 *
 * @param error The error message.
 *
 * @returns An async symbol schema.
 */
declare function symbolAsync(error?: ErrorMessage): SymbolSchemaAsync;

/**
 * Tuple shape async type.
 */
type TupleShapeAsync = [
    BaseSchema | BaseSchemaAsync,
    ...(BaseSchema[] | BaseSchemaAsync[])
];
/**
 * Tuple schema async type.
 */
type TupleSchemaAsync<TTupleItems extends TupleShapeAsync, TTupleRest extends BaseSchema | BaseSchemaAsync | undefined = undefined, TOutput = TupleOutput<TTupleItems, TTupleRest>> = BaseSchemaAsync<TupleInput<TTupleItems, TTupleRest>, TOutput> & {
    schema: 'tuple';
    tuple: {
        items: TTupleItems;
        rest: TTupleRest;
    };
};
/**
 * Creates an async tuple schema.
 *
 * @param items The items schema.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An async tuple schema.
 */
declare function tupleAsync<TTupleItems extends TupleShapeAsync>(items: TTupleItems, pipe?: PipeAsync<TupleOutput<TTupleItems, undefined>>): TupleSchemaAsync<TTupleItems>;
/**
 * Creates an async tuple schema.
 *
 * @param items The items schema.
 * @param error The error message.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An async tuple schema.
 */
declare function tupleAsync<TTupleItems extends TupleShapeAsync>(items: TTupleItems, error?: ErrorMessage, pipe?: PipeAsync<TupleOutput<TTupleItems, undefined>>): TupleSchemaAsync<TTupleItems>;
/**
 * Creates an async tuple schema.
 *
 * @param items The items schema.
 * @param rest The rest schema.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An async tuple schema.
 */
declare function tupleAsync<TTupleItems extends TupleShapeAsync, TTupleRest extends BaseSchema | BaseSchemaAsync | undefined>(items: TTupleItems, rest: TTupleRest, pipe?: PipeAsync<TupleOutput<TTupleItems, TTupleRest>>): TupleSchemaAsync<TTupleItems, TTupleRest>;
/**
 * Creates an async tuple schema.
 *
 * @param items The items schema.
 * @param rest The rest schema.
 * @param error The error message.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An async tuple schema.
 */
declare function tupleAsync<TTupleItems extends TupleShapeAsync, TTupleRest extends BaseSchema | BaseSchemaAsync | undefined>(items: TTupleItems, rest: TTupleRest, error?: ErrorMessage, pipe?: PipeAsync<TupleOutput<TTupleItems, TTupleRest>>): TupleSchemaAsync<TTupleItems, TTupleRest>;

/**
 * Tuple path item type.
 */
type TuplePathItem = {
    schema: 'tuple';
    input: [any, ...any[]];
    key: number;
    value: any;
};
/**
 * Tuple input inference type.
 */
type TupleInput<TTupleItems extends TupleShape | TupleShapeAsync, TTupleRest extends BaseSchema | BaseSchemaAsync | undefined> = TTupleRest extends BaseSchema | BaseSchemaAsync ? [
    ...{
        [TKey in keyof TTupleItems]: Input<TTupleItems[TKey]>;
    },
    ...Input<TTupleRest>[]
] : {
    [TKey in keyof TTupleItems]: Input<TTupleItems[TKey]>;
};
/**
 * Tuple with rest output inference type.
 */
type TupleOutput<TTupleItems extends TupleShape | TupleShapeAsync, TTupleRest extends BaseSchema | BaseSchemaAsync | undefined> = TTupleRest extends BaseSchema | BaseSchemaAsync ? [
    ...{
        [TKey in keyof TTupleItems]: Output<TTupleItems[TKey]>;
    },
    ...Output<TTupleRest>[]
] : {
    [TKey in keyof TTupleItems]: Output<TTupleItems[TKey]>;
};

/**
 * Tuple shape type.
 */
type TupleShape = [BaseSchema, ...BaseSchema[]];
/**
 * Tuple schema type.
 */
type TupleSchema<TTupleItems extends TupleShape, TTupleRest extends BaseSchema | undefined = undefined, TOutput = TupleOutput<TTupleItems, TTupleRest>> = BaseSchema<TupleInput<TTupleItems, TTupleRest>, TOutput> & {
    schema: 'tuple';
    tuple: {
        items: TTupleItems;
        rest: TTupleRest;
    };
};
/**
 * Creates a tuple schema.
 *
 * @param items The items schema.
 * @param pipe A validation and transformation pipe.
 *
 * @returns A tuple schema.
 */
declare function tuple<TTupleItems extends TupleShape>(items: TTupleItems, pipe?: Pipe<TupleOutput<TTupleItems, undefined>>): TupleSchema<TTupleItems>;
/**
 * Creates a tuple schema.
 *
 * @param items The items schema.
 * @param error The error message.
 * @param pipe A validation and transformation pipe.
 *
 * @returns A tuple schema.
 */
declare function tuple<TTupleItems extends TupleShape>(items: TTupleItems, error?: ErrorMessage, pipe?: Pipe<TupleOutput<TTupleItems, undefined>>): TupleSchema<TTupleItems>;
/**
 * Creates a tuple schema.
 *
 * @param items The items schema.
 * @param rest The rest schema.
 * @param pipe A validation and transformation pipe.
 *
 * @returns A tuple schema.
 */
declare function tuple<TTupleItems extends TupleShape, TTupleRest extends BaseSchema>(items: TTupleItems, rest: TTupleRest, pipe?: Pipe<TupleOutput<TTupleItems, TTupleRest>>): TupleSchema<TTupleItems, TTupleRest>;
/**
 * Creates a tuple schema.
 *
 * @param items The items schema.
 * @param rest The rest schema.
 * @param error The error message.
 * @param pipe A validation and transformation pipe.
 *
 * @returns A tuple schema.
 */
declare function tuple<TTupleItems extends TupleShape, TTupleRest extends BaseSchema>(items: TTupleItems, rest: TTupleRest, error?: ErrorMessage, pipe?: Pipe<TupleOutput<TTupleItems, TTupleRest>>): TupleSchema<TTupleItems, TTupleRest>;

/**
 * Undefined schema type.
 */
type UndefinedSchema<TOutput = undefined> = BaseSchema<undefined, TOutput> & {
    schema: 'undefined';
};
/**
 * Creates a undefined schema.
 *
 * @param error The error message.
 *
 * @returns A undefined schema.
 */
declare function undefinedType(error?: ErrorMessage): UndefinedSchema;

/**
 * Undefined schema async type.
 */
type UndefinedSchemaAsync<TOutput = undefined> = BaseSchemaAsync<undefined, TOutput> & {
    schema: 'undefined';
};
/**
 * Creates an async undefined schema.
 *
 * @param error The error message.
 *
 * @returns An async undefined schema.
 */
declare function undefinedTypeAsync(error?: ErrorMessage): UndefinedSchemaAsync;

/**
 * Union options type.
 */
type UnionOptions = [
    BaseSchema<any>,
    BaseSchema<any>,
    ...BaseSchema<any>[]
];
/**
 * Union schema type.
 */
type UnionSchema<TUnionOptions extends UnionOptions, TOutput = Output<TUnionOptions[number]>> = BaseSchema<Input<TUnionOptions[number]>, TOutput> & {
    schema: 'union';
    union: TUnionOptions;
};
/**
 * Creates a union schema.
 *
 * @param union The union schema.
 * @param error The error message.
 *
 * @returns A union schema.
 */
declare function union<TUnionOptions extends UnionOptions>(union: TUnionOptions, error?: ErrorMessage): UnionSchema<TUnionOptions>;

/**
 * Union options async type.
 */
type UnionOptionsAsync = [
    BaseSchema | BaseSchemaAsync,
    BaseSchema | BaseSchemaAsync,
    ...(BaseSchema[] | BaseSchemaAsync[])
];
/**
 * Union schema async type.
 */
type UnionSchemaAsync<TUnionOptions extends UnionOptionsAsync, TOutput = Output<TUnionOptions[number]>> = BaseSchemaAsync<Input<TUnionOptions[number]>, TOutput> & {
    schema: 'union';
    union: TUnionOptions;
};
/**
 * Creates an async union schema.
 *
 * @param union The union schema.
 * @param error The error message.
 *
 * @returns An async union schema.
 */
declare function unionAsync<TUnionOptions extends UnionOptionsAsync>(union: TUnionOptions, error?: ErrorMessage): UnionSchemaAsync<TUnionOptions>;

/**
 * Unknown schema type.
 */
type UnknownSchema<TOutput = unknown> = BaseSchema<unknown, TOutput> & {
    schema: 'unknown';
};
/**
 * Creates a unknown schema.
 *
 * @param pipe A validation and transformation pipe.
 *
 * @returns A unknown schema.
 */
declare function unknown(pipe?: Pipe<unknown>): UnknownSchema;

/**
 * Unknown schema async type.
 */
type UnknownSchemaAsync<TOutput = unknown> = BaseSchemaAsync<unknown, TOutput> & {
    schema: 'unknown';
};
/**
 * Creates an async unknown schema.
 *
 * @param pipe A validation and transformation pipe.
 *
 * @returns An async unknown schema.
 */
declare function unknownAsync(pipe?: PipeAsync<unknown>): UnknownSchemaAsync;

/**
 * Void schema type.
 */
type VoidSchema<TOutput = void> = BaseSchema<void, TOutput> & {
    schema: 'void';
};
/**
 * Creates a void schema.
 *
 * @param error The error message.
 *
 * @returns A void schema.
 */
declare function voidType(error?: ErrorMessage): VoidSchema;

/**
 * Void schema async type.
 */
type VoidSchemaAsync<TOutput = void> = BaseSchemaAsync<void, TOutput> & {
    schema: 'void';
};
/**
 * Creates an async void schema.
 *
 * @param error The error message.
 *
 * @returns An async void schema.
 */
declare function voidTypeAsync(error?: ErrorMessage): VoidSchemaAsync;

/**
 * Issue reason type.
 */
type IssueReason = 'type' | 'string' | 'number' | 'bigint' | 'blob' | 'boolean' | 'any' | 'unknown' | 'date' | 'array' | 'tuple' | 'map' | 'object' | 'record' | 'set' | 'special' | 'instance';
/**
 * Issue origin type.
 */
type IssueOrigin = 'key' | 'value';
/**
 * Issue type.
 */
type Issue = {
    reason: IssueReason;
    validation: string;
    origin: IssueOrigin;
    message: string;
    input: any;
    path?: PathItem[];
    issues?: Issues;
    abortEarly?: boolean;
    abortPipeEarly?: boolean;
    skipPipe?: boolean;
};
/**
 * Issues type.
 */
type Issues = [Issue, ...Issue[]];
/**
 * Parse info type.
 */
type ParseInfo = Partial<Pick<Issue, 'origin' | 'abortEarly' | 'abortPipeEarly' | 'skipPipe'>>;
/**
 * Path item type.
 */
type PathItem = ObjectPathItem | RecordPathItem | TuplePathItem | MapPathItem | SetPathItem | ArrayPathItem;
/**
 * Parse result type.
 */
type _ParseResult<TOutput> = {
    output: TOutput;
    issues?: undefined;
} | {
    output?: undefined;
    issues: Issues;
};
/**
 * Base schema type.
 */
type BaseSchema<TInput = any, TOutput = TInput> = {
    async: false;
    _parse(input: unknown, info?: ParseInfo): _ParseResult<TOutput>;
    _types?: {
        input: TInput;
        output: TOutput;
    };
};
/**
 * Base schema async type.
 */
type BaseSchemaAsync<TInput = any, TOutput = TInput> = {
    async: true;
    _parse(input: unknown, info?: ParseInfo): Promise<_ParseResult<TOutput>>;
    _types?: {
        input: TInput;
        output: TOutput;
    };
};
/**
 * Input inference type.
 */
type Input<TSchema extends BaseSchema | BaseSchemaAsync> = NonNullable<TSchema['_types']>['input'];
/**
 * Output inference type.
 */
type Output<TSchema extends BaseSchema | BaseSchemaAsync> = NonNullable<TSchema['_types']>['output'];
/**
 * Error message type.
 */
type ErrorMessage = string | (() => string);
/**
 * Pipe info type.
 */
type PipeInfo = ParseInfo & Pick<Issue, 'reason'>;
/**
 * Pipe result type.
 */
type PipeResult<TOutput> = {
    output: TOutput;
    issues?: undefined;
} | {
    output?: undefined;
    issues: Pick<Issue, 'validation' | 'message' | 'input' | 'path'>[];
};
/**
 * Validation and transformation pipe type.
 */
type Pipe<TValue> = ((value: TValue) => PipeResult<TValue>)[];
/**
 * Async validation and transformation pipe type.
 */
type PipeAsync<TValue> = ((value: TValue) => PipeResult<TValue> | Promise<PipeResult<TValue>>)[];
/**
 * Resolve type.
 *
 * Hint: This type has no effect and is only used so that TypeScript displays
 * the final type in the preview instead of the utility types used.
 */
type Resolve<T> = T;
/**
 * Resolve object type.
 *
 * Hint: This type has no effect and is only used so that TypeScript displays
 * the final type in the preview instead of the utility types used.
 */
type ResolveObject<T> = Resolve<{
    [k in keyof T]: T[k];
}>;

/**
 * A Valibot error with useful information.
 */
declare class ValiError extends Error {
    issues: Issues;
    /**
     * Creates a Valibot error with useful information.
     *
     * @param issues The error issues.
     */
    constructor(issues: Issues);
}

/**
 * Flat errors type.
 */
type FlatErrors = {
    root?: [string, ...string[]];
    nested: Partial<Record<string, [string, ...string[]]>>;
};
/**
 * Flatten the error messages of a Vali error.
 *
 * @param error A Vali error.
 *
 * @returns Flat errors.
 */
declare function flatten(error: ValiError): FlatErrors;
/**
 * Flatten the error messages of issues.
 *
 * @param issues The issues.
 *
 * @returns Flat errors.
 */
declare function flatten(issues: Issues): FlatErrors;

declare const BrandSymbol: unique symbol;
/**
 * Brand name type.
 */
type BrandName = string | number | symbol;
/**
 * Brand type.
 */
type Brand<TBrandName extends BrandName> = {
    [BrandSymbol]: {
        [TValue in TBrandName]: TValue;
    };
};
declare function brand<TSchema extends AnySchema, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): AnySchema<Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends AnySchemaAsync, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): AnySchemaAsync<Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends ArraySchema<any>, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): ArraySchema<TSchema['array']['item'], Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends ArraySchemaAsync<any>, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): ArraySchemaAsync<TSchema['array']['item'], Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends BigintSchema, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): BigintSchema<Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends BigintSchemaAsync, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): BigintSchemaAsync<Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends BlobSchema, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): BlobSchema<Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends BlobSchemaAsync, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): BlobSchemaAsync<Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends BooleanSchema, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): BooleanSchema<Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends BooleanSchemaAsync, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): BooleanSchemaAsync<Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends DateSchema, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): DateSchema<Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends DateSchemaAsync, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): DateSchemaAsync<Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends EnumSchema<any>, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): EnumSchema<TSchema['enum'], Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends EnumSchemaAsync<any>, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): EnumSchemaAsync<TSchema['enum'], Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends InstanceSchema<any>, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): InstanceSchema<TSchema['class'], Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends InstanceSchemaAsync<any>, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): InstanceSchemaAsync<TSchema['class'], Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends LiteralSchema<any>, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): LiteralSchema<TSchema['literal'], Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends LiteralSchemaAsync<any>, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): LiteralSchemaAsync<TSchema['literal'], Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends MapSchema<any, any>, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): MapSchema<TSchema['map']['key'], TSchema['map']['value'], Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends MapSchemaAsync<any, any>, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): MapSchemaAsync<TSchema['map']['key'], TSchema['map']['value'], Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends NanSchema, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): NanSchema<Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends NanSchemaAsync, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): NanSchemaAsync<Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends NativeEnumSchema<any>, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): NativeEnumSchema<TSchema['nativeEnum'], Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends NativeEnumSchemaAsync<any>, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): NativeEnumSchemaAsync<TSchema['nativeEnum'], Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends NeverSchema>(schema: TSchema, action: (value: Output<TSchema>) => never): NeverSchema;
declare function brand<TSchema extends NeverSchemaAsync>(schema: TSchema, action: (value: Output<TSchema>) => never): NeverSchemaAsync;
declare function brand<TSchema extends NonNullableSchema<any>, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): NonNullableSchema<TSchema['wrapped'], Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends NonNullableSchemaAsync<any>, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): NonNullableSchemaAsync<TSchema['wrapped'], Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends NonNullishSchema<any>, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): NonNullishSchema<TSchema['wrapped'], Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends NonNullishSchemaAsync<any>, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): NonNullishSchemaAsync<TSchema['wrapped'], Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends NonOptionalSchema<any>, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): NonOptionalSchema<TSchema['wrapped'], Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends NonOptionalSchemaAsync<any>, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): NonOptionalSchemaAsync<TSchema['wrapped'], Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends NullSchema, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): NullSchema<Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends NullSchemaAsync, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): NullSchemaAsync<Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends NullableSchema<any>, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): NullableSchema<TSchema['wrapped'], Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends NullableSchemaAsync<any>, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): NullableSchemaAsync<TSchema['wrapped'], Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends NullishSchema<any>, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): NullishSchema<TSchema['wrapped'], Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends NullishSchemaAsync<any>, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): NullishSchemaAsync<TSchema['wrapped'], Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends NumberSchema, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): NumberSchema<Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends NumberSchemaAsync, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): NumberSchemaAsync<Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends ObjectSchema<any>, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): ObjectSchema<TSchema['object'], Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends ObjectSchemaAsync<any>, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): ObjectSchemaAsync<TSchema['object'], Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends OptionalSchema<any>, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): OptionalSchema<TSchema['wrapped'], Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends OptionalSchemaAsync<any>, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): OptionalSchemaAsync<TSchema['wrapped'], Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends RecordSchema<any, any>, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): RecordSchema<TSchema['record']['key'], TSchema['record']['value'], Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends RecordSchemaAsync<any, any>, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): RecordSchemaAsync<TSchema['record']['key'], TSchema['record']['value'], Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends RecursiveSchema<any>, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): RecursiveSchema<TSchema['getter'], Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends RecursiveSchemaAsync<any>, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): RecursiveSchemaAsync<TSchema['getter'], Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends SetSchema<any>, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): SetSchema<TSchema['set']['value'], Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends SetSchemaAsync<any>, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): SetSchemaAsync<TSchema['set']['value'], Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends SpecialSchema<any>, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): SpecialSchema<Input<TSchema>, Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends SpecialSchemaAsync<any>, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): SpecialSchemaAsync<Input<TSchema>, Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends StringSchema, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): StringSchema<Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends StringSchemaAsync, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): StringSchemaAsync<Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends SymbolSchema, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): SymbolSchema<Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends SymbolSchemaAsync, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): SymbolSchemaAsync<Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends TupleSchema<any, any>, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): TupleSchema<TSchema['tuple']['items'], TSchema['tuple']['rest'], Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends TupleSchemaAsync<any, any>, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): TupleSchemaAsync<TSchema['tuple']['items'], TSchema['tuple']['rest'], Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends UndefinedSchema, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): UndefinedSchema<Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends UndefinedSchemaAsync, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): UndefinedSchemaAsync<Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends UnionSchema<any>, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): UnionSchema<TSchema['union'], Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends UnionSchemaAsync<any>, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): UnionSchemaAsync<TSchema['union'], Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends UnknownSchema, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): UnknownSchema<Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends UnknownSchemaAsync, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): UnknownSchemaAsync<Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends VoidSchema, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): VoidSchema<Output<TSchema> & Brand<TBrandName>>;
declare function brand<TSchema extends VoidSchemaAsync, TBrandName extends BrandName>(schema: TSchema, name: TBrandName): VoidSchemaAsync<Output<TSchema> & Brand<TBrandName>>;

/**
 * Coerces the input of a schema to match the required type.
 *
 * @param schema The affected schema.
 * @param action The coerceation action.
 *
 * @returns The passed schema.
 */
declare function coerce<TSchema extends BaseSchema>(schema: TSchema, action: (value: unknown) => Input<TSchema>): TSchema;

/**
 * Coerces the input of a async schema to match the required type.
 *
 * @param schema The affected schema.
 * @param action The coerceation action.
 *
 * @returns The passed schema.
 */
declare function coerceAsync<TSchema extends BaseSchemaAsync>(schema: TSchema, action: (value: unknown) => Input<TSchema> | Promise<Input<TSchema>>): TSchema;

/**
 * Fallback info type.
 */
type FallbackInfo = {
    input: unknown;
    issues: Issues;
};

/**
 * Returns a fallback value when validating the passed schema failed.
 *
 * @param schema The schema to catch.
 * @param value The fallback value.
 *
 * @returns The passed schema.
 */
declare function fallback<TSchema extends BaseSchema>(schema: TSchema, value: Output<TSchema> | ((info: FallbackInfo) => Output<TSchema>)): TSchema;

/**
 * Returns a fallback value when validating the passed schema failed.
 *
 * @param schema The schema to catch.
 * @param value The fallback value.
 *
 * @returns The passed schema.
 */
declare function fallbackAsync<TSchema extends BaseSchema | BaseSchemaAsync>(schema: TSchema, value: Output<TSchema> | ((info: FallbackInfo) => Output<TSchema>)): TSchema;

/**
 * Returns the default value of the schema.
 *
 * @param schema The schema to get the default value from.
 *
 * @returns The default value.
 */
declare function getDefault<TSchema extends OptionalSchema<any, any> | OptionalSchemaAsync<any, any> | NullableSchema<any, any> | NullableSchemaAsync<any, any> | NullishSchema<any, any> | NullishSchemaAsync<any, any>>(schema: TSchema): TSchema['default'];

/**
 * Checks if the input matches the scheme. By using a type predicate, this
 * function can be used as a type guard.
 *
 * @param schema The schema to be used.
 * @param input The input to be tested.
 *
 * @returns Whether the input matches the scheme.
 */
declare function is<TSchema extends BaseSchema>(schema: TSchema, input: unknown): input is Input<TSchema>;

/**
 * Converts union to intersection types.
 */
type UnionToIntersection<T> = (T extends never ? never : (arg: T) => never) extends (arg: infer U) => never ? U : never;
/**
 * Converts union to tuple types.
 */
type UnionToTuple<T> = UnionToIntersection<T extends never ? never : () => T> extends () => infer W ? [...UnionToTuple<Exclude<T, W>>, W] : [];
/**
 * Returns a tuple or never type.
 */
type TupleOrNever<T> = T extends [string, ...string[]] ? T : never;
/**
 * Creates a enum schema of object keys.
 *
 * @param schema The object schema.
 *
 * @returns A enum schema.
 */
declare function keyof<TSchema extends ObjectSchema<any> | ObjectSchemaAsync<any>>(schema: TSchema): EnumSchema<TupleOrNever<UnionToTuple<keyof TSchema['object']>>>;

/**
 * Merges schema objects types.
 */
type MergeSchemaObjects<TObjectSchemas extends (ObjectSchema<any> | ObjectSchemaAsync<any>)[]> = TObjectSchemas extends [infer TFirstObjectSchema] ? TFirstObjectSchema extends ObjectSchema<any> | ObjectSchemaAsync<any> ? TFirstObjectSchema['object'] : never : TObjectSchemas extends [
    infer TFirstObjectSchema,
    ...infer TRestObjectSchemas
] ? TFirstObjectSchema extends ObjectSchema<any> | ObjectSchemaAsync<any> ? TRestObjectSchemas extends (ObjectSchema<any> | ObjectSchemaAsync<any>)[] ? {
    [TKey in Exclude<keyof TFirstObjectSchema['object'], keyof MergeSchemaObjects<TRestObjectSchemas>>]: TFirstObjectSchema['object'][TKey];
} & MergeSchemaObjects<TRestObjectSchemas> : never : never : never;

/**
 * Object schemas type.
 */
type ObjectSchemas$1 = [
    ObjectSchema<any>,
    ObjectSchema<any>,
    ...ObjectSchema<any>[]
];
/**
 * Merges multiple object schemas into a single one. Subsequent object schemas
 * overwrite the previous ones.
 *
 * @param schemas The schemas to be merged.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An object schema.
 */
declare function merge<TObjectSchemas extends ObjectSchemas$1>(schemas: TObjectSchemas, pipe?: Pipe<ObjectOutput<MergeSchemaObjects<TObjectSchemas>>>): ObjectSchema<MergeSchemaObjects<TObjectSchemas>>;
/**
 * Merges multiple object schemas into a single one. Subsequent object schemas
 * overwrite the previous ones.
 *
 * @param schemas The schemas to be merged.
 * @param error The error message.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An object schema.
 */
declare function merge<TObjectSchemas extends ObjectSchemas$1>(schemas: TObjectSchemas, error?: ErrorMessage, pipe?: Pipe<ObjectOutput<MergeSchemaObjects<TObjectSchemas>>>): ObjectSchema<MergeSchemaObjects<TObjectSchemas>>;

/**
 * Object schemas type.
 */
type ObjectSchemas = [
    ObjectSchema<any> | ObjectSchemaAsync<any>,
    ObjectSchema<any> | ObjectSchemaAsync<any>,
    ...(ObjectSchema<any> | ObjectSchemaAsync<any>)[]
];
/**
 * Merges multiple async object schemas into a single one. Subsequent object
 * schemas overwrite the previous ones.
 *
 * @param schemas The schemas to be merged.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An async object schema.
 */
declare function mergeAsync<TObjectSchemas extends ObjectSchemas>(schemas: TObjectSchemas, pipe?: PipeAsync<ObjectOutput<MergeSchemaObjects<TObjectSchemas>>>): ObjectSchemaAsync<MergeSchemaObjects<TObjectSchemas>>;
/**
 * Merges multiple async object schemas into a single one. Subsequent object
 * schemas overwrite the previous ones.
 *
 * @param schemas The schemas to be merged.
 * @param error The error message.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An async object schema.
 */
declare function mergeAsync<TObjectSchemas extends ObjectSchemas>(schemas: TObjectSchemas, error?: ErrorMessage, pipe?: PipeAsync<ObjectOutput<MergeSchemaObjects<TObjectSchemas>>>): ObjectSchemaAsync<MergeSchemaObjects<TObjectSchemas>>;

/**
 * Object keys type.
 */
type ObjectKeys<TObjectSchema extends ObjectSchema<any> | ObjectSchemaAsync<any>> = [keyof TObjectSchema['object'], ...(keyof TObjectSchema['object'])[]];

/**
 * Creates an object schema that contains not the selected keys of an existing
 * schema.
 *
 * @param schema The schema to omit from.
 * @param keys The selected keys
 * @param pipe A validation and transformation pipe.
 *
 * @returns An object schema.
 */
declare function omit<TObjectSchema extends ObjectSchema<any>, TKeys extends ObjectKeys<TObjectSchema>>(schema: TObjectSchema, keys: TKeys, pipe?: Pipe<ObjectOutput<Omit<TObjectSchema['object'], TKeys[number]>>>): ObjectSchema<Omit<TObjectSchema['object'], TKeys[number]>>;
/**
 * Creates an object schema that contains not the selected keys of an existing
 * schema.
 *
 * @param schema The schema to omit from.
 * @param keys The selected keys
 * @param error The error message.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An object schema.
 */
declare function omit<TObjectSchema extends ObjectSchema<any>, TKeys extends ObjectKeys<TObjectSchema>>(schema: TObjectSchema, keys: TKeys, error?: ErrorMessage, pipe?: Pipe<ObjectOutput<Omit<TObjectSchema['object'], TKeys[number]>>>): ObjectSchema<Omit<TObjectSchema['object'], TKeys[number]>>;

/**
 * Creates an async object schema that contains only the selected keys of an
 * existing schema.
 *
 * @param schema The schema to omit from.
 * @param keys The selected keys
 * @param pipe A validation and transformation pipe.
 *
 * @returns An async object schema.
 */
declare function omitAsync<TObjectSchema extends ObjectSchema<any> | ObjectSchemaAsync<any>, TKeys extends ObjectKeys<TObjectSchema>>(schema: TObjectSchema, keys: TKeys, pipe?: PipeAsync<ObjectOutput<Omit<TObjectSchema['object'], TKeys[number]>>>): ObjectSchemaAsync<Omit<TObjectSchema['object'], TKeys[number]>>;
/**
 * Creates an async object schema that contains only the selected keys of an
 * existing schema.
 *
 * @param schema The schema to omit from.
 * @param keys The selected keys
 * @param error The error message.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An async object schema.
 */
declare function omitAsync<TObjectSchema extends ObjectSchema<any> | ObjectSchemaAsync<any>, TKeys extends ObjectKeys<TObjectSchema>>(schema: TObjectSchema, keys: TKeys, error?: ErrorMessage, pipe?: PipeAsync<ObjectOutput<Omit<TObjectSchema['object'], TKeys[number]>>>): ObjectSchemaAsync<Omit<TObjectSchema['object'], TKeys[number]>>;

/**
 * Parses unknown input based on a schema.
 *
 * @param schema The schema to be used.
 * @param input The input to be parsed.
 * @param info The optional parse info.
 *
 * @returns The parsed output.
 */
declare function parse<TSchema extends BaseSchema>(schema: TSchema, input: unknown, info?: Pick<ParseInfo, 'abortEarly' | 'abortPipeEarly' | 'skipPipe'>): Output<TSchema>;

/**
 * Parses unknown input based on a schema.
 *
 * @param schema The schema to be used.
 * @param input The input to be parsed.
 * @param info The optional parse info.
 *
 * @returns The parsed output.
 */
declare function parseAsync<TSchema extends BaseSchema | BaseSchemaAsync>(schema: TSchema, input: unknown, info?: Pick<ParseInfo, 'abortEarly' | 'abortPipeEarly' | 'skipPipe'>): Promise<Output<TSchema>>;

/**
 * Partial object schema type.
 */
type Partial$2<TObjectShape extends ObjectShape> = {
    [TKey in keyof TObjectShape]: OptionalSchema<TObjectShape[TKey]>;
};
/**
 * Creates an object schema consisting of all properties of an existing object
 * schema set to optional.
 *
 * @param schema The affected schema.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An object schema.
 */
declare function partial<TObjectSchema extends ObjectSchema<any>>(schema: TObjectSchema, pipe?: Pipe<ObjectOutput<Partial$2<TObjectSchema['object']>>>): ObjectSchema<Partial$2<TObjectSchema['object']>>;
/**
 * Creates an object schema consisting of all properties of an existing object
 * schema set to optional.
 *
 * @param schema The affected schema.
 * @param error The error message.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An object schema.
 */
declare function partial<TObjectSchema extends ObjectSchema<any>>(schema: TObjectSchema, error?: ErrorMessage, pipe?: Pipe<ObjectOutput<Partial$2<TObjectSchema['object']>>>): ObjectSchema<Partial$2<TObjectSchema['object']>>;

/**
 * Partial object schema type.
 */
type Partial$1<TObjectShape extends ObjectShapeAsync> = {
    [TKey in keyof TObjectShape]: OptionalSchemaAsync<TObjectShape[TKey]>;
};
/**
 * Creates an async object schema consisting of all properties of an existing
 * object schema set to optional.
 *
 * @param schema The affected schema.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An async object schema.
 */
declare function partialAsync<TObjectSchema extends ObjectSchema<any> | ObjectSchemaAsync<any>>(schema: TObjectSchema, pipe?: PipeAsync<ObjectOutput<Partial$1<TObjectSchema['object']>>>): ObjectSchemaAsync<Partial$1<TObjectSchema['object']>>;
/**
 * Creates an async object schema consisting of all properties of an existing
 * object schema set to optional.
 *
 * @param schema The affected schema.
 * @param error The error message.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An async object schema.
 */
declare function partialAsync<TObjectSchema extends ObjectSchema<any> | ObjectSchemaAsync<any>>(schema: TObjectSchema, error?: ErrorMessage, pipe?: PipeAsync<ObjectOutput<Partial$1<TObjectSchema['object']>>>): ObjectSchemaAsync<Partial$1<TObjectSchema['object']>>;

/**
 * Creates an object schema that passes unknown keys.
 *
 * @param schema A object schema.
 *
 * @returns A object schema.
 */
declare function passthrough<TSchema extends ObjectSchema<any>>(schema: TSchema): TSchema;

/**
 * Creates an object schema that passes unknown keys.
 *
 * @param schema A object schema.
 *
 * @returns A object schema.
 */
declare function passthroughAsync<TSchema extends ObjectSchemaAsync<any>>(schema: TSchema): TSchema;

/**
 * Creates an object schema that contains only the selected keys of an existing
 * schema.
 *
 * @param schema The schema to pick from.
 * @param keys The selected keys
 * @param pipe A validation and transformation pipe.
 *
 * @returns An object schema.
 */
declare function pick<TObjectSchema extends ObjectSchema<any>, TKeys extends (keyof TObjectSchema['object'])[]>(schema: TObjectSchema, keys: TKeys, pipe?: Pipe<ObjectOutput<Pick<TObjectSchema['object'], TKeys[number]>>>): ObjectSchema<Pick<TObjectSchema['object'], TKeys[number]>>;
/**
 * Creates an object schema that contains only the selected keys of an existing
 * schema.
 *
 * @param schema The schema to pick from.
 * @param keys The selected keys
 * @param error The error message.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An object schema.
 */
declare function pick<TObjectSchema extends ObjectSchema<any>, TKeys extends (keyof TObjectSchema['object'])[]>(schema: TObjectSchema, keys: TKeys, error?: ErrorMessage, pipe?: Pipe<ObjectOutput<Pick<TObjectSchema['object'], TKeys[number]>>>): ObjectSchema<Pick<TObjectSchema['object'], TKeys[number]>>;

/**
 * Creates an async object schema that contains only the selected keys of an
 * existing schema.
 *
 * @param schema The schema to pick from.
 * @param keys The selected keys
 * @param pipe A validation and transformation pipe.
 *
 * @returns An async object schema.
 */
declare function pickAsync<TObjectSchema extends ObjectSchema<any> | ObjectSchemaAsync<any>, TKeys extends (keyof TObjectSchema['object'])[]>(schema: TObjectSchema, keys: TKeys, pipe?: PipeAsync<ObjectOutput<Pick<TObjectSchema['object'], TKeys[number]>>>): ObjectSchemaAsync<Pick<TObjectSchema['object'], TKeys[number]>>;
/**
 * Creates an async object schema that contains only the selected keys of an
 * existing schema.
 *
 * @param schema The schema to pick from.
 * @param keys The selected keys
 * @param error The error message.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An async object schema.
 */
declare function pickAsync<TObjectSchema extends ObjectSchema<any> | ObjectSchemaAsync<any>, TKeys extends (keyof TObjectSchema['object'])[]>(schema: TObjectSchema, keys: TKeys, error?: ErrorMessage, pipe?: PipeAsync<ObjectOutput<Pick<TObjectSchema['object'], TKeys[number]>>>): ObjectSchemaAsync<Pick<TObjectSchema['object'], TKeys[number]>>;

/**
 * Required object schema type.
 */
type Required$1<TObjectShape extends ObjectShape> = {
    [TKey in keyof TObjectShape]: NonOptionalSchema<TObjectShape[TKey]>;
};
/**
 * Creates an object schema consisting of all properties of an existing object
 * schema set to none optional.
 *
 * @param schema The affected schema.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An object schema.
 */
declare function required<TObjectSchema extends ObjectSchema<any>>(schema: TObjectSchema, pipe?: Pipe<ObjectOutput<Required$1<TObjectSchema['object']>>>): ObjectSchema<Required$1<TObjectSchema['object']>>;
/**
 * Creates an object schema consisting of all properties of an existing object
 * schema set to none optional.
 *
 * @param schema The affected schema.
 * @param error The error message.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An object schema.
 */
declare function required<TObjectSchema extends ObjectSchema<any>>(schema: TObjectSchema, error?: ErrorMessage, pipe?: Pipe<ObjectOutput<Required$1<TObjectSchema['object']>>>): ObjectSchema<Required$1<TObjectSchema['object']>>;

/**
 * Required object schema type.
 */
type Required<TObjectShape extends ObjectShapeAsync> = {
    [TKey in keyof TObjectShape]: NonOptionalSchemaAsync<TObjectShape[TKey]>;
};
/**
 * Creates an async object schema consisting of all properties of an existing
 * object schema set to none optional.
 *
 * @param schema The affected schema.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An async object schema.
 */
declare function requiredAsync<TObjectSchema extends ObjectSchema<any> | ObjectSchemaAsync<any>>(schema: TObjectSchema, pipe?: PipeAsync<ObjectOutput<Required<TObjectSchema['object']>>>): ObjectSchemaAsync<Required<TObjectSchema['object']>>;
/**
 * Creates an async object schema consisting of all properties of an existing
 * object schema set to none optional.
 *
 * @param schema The affected schema.
 * @param error The error message.
 * @param pipe A validation and transformation pipe.
 *
 * @returns An async object schema.
 */
declare function requiredAsync<TObjectSchema extends ObjectSchema<any> | ObjectSchemaAsync<any>>(schema: TObjectSchema, error?: ErrorMessage, pipe?: PipeAsync<ObjectOutput<Required<TObjectSchema['object']>>>): ObjectSchemaAsync<Required<TObjectSchema['object']>>;

/**
 * Safe parse result type.
 */
type SafeParseResult<TSchema extends BaseSchema | BaseSchemaAsync> = {
    success: true;
    /**
     * @deprecated Please use `.output` instead.
     */
    data: Output<TSchema>;
    output: Output<TSchema>;
} | {
    success: false;
    /**
     * @deprecated Please use `.issues` instead.
     */
    error: ValiError;
    issues: Issues;
};

/**
 * Parses unknown input based on a schema.
 *
 * @param schema The schema to be used.
 * @param input The input to be parsed.
 * @param info The optional parse info.
 *
 * @returns The parsed output.
 */
declare function safeParse<TSchema extends BaseSchema>(schema: TSchema, input: unknown, info?: Pick<ParseInfo, 'abortEarly' | 'abortPipeEarly' | 'skipPipe'>): SafeParseResult<TSchema>;

/**
 * Parses unknown input based on a schema.
 *
 * @param schema The schema to be used.
 * @param input The input to be parsed.
 * @param info The optional parse info.
 *
 * @returns The parsed output.
 */
declare function safeParseAsync<TSchema extends BaseSchema | BaseSchemaAsync>(schema: TSchema, input: unknown, info?: Pick<ParseInfo, 'abortEarly' | 'abortPipeEarly' | 'skipPipe'>): Promise<SafeParseResult<TSchema>>;

/**
 * Creates a strict object schema that throws an error if an input contains
 * unknown keys.
 *
 * @param schema A object schema.
 * @param error The error message.
 *
 * @returns A strict object schema.
 */
declare function strict<TSchema extends ObjectSchema<any>>(schema: TSchema, error?: ErrorMessage): TSchema;

/**
 * Creates a strict async object schema that throws an error if an input
 * contains unknown keys.
 *
 * @param schema A object schema.
 * @param error The error message.
 *
 * @returns A strict object schema.
 */
declare function strictAsync<TSchema extends ObjectSchemaAsync<any>>(schema: TSchema, error?: ErrorMessage): TSchema;

/**
 * Creates an object schema that strips unknown keys.
 *
 * @param schema A object schema.
 *
 * @returns A object schema.
 */
declare function strip<TSchema extends ObjectSchema<any>>(schema: TSchema): TSchema;

/**
 * Creates an object schema that strips unknown keys.
 *
 * @param schema A object schema.
 *
 * @returns A object schema.
 */
declare function stripAsync<TSchema extends ObjectSchemaAsync<any>>(schema: TSchema): TSchema;

declare function transform<TSchema extends AnySchema, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput): AnySchema<TOutput>;
declare function transform<TSchema extends ArraySchema<any>, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput): ArraySchema<TSchema['array']['item'], TOutput>;
declare function transform<TSchema extends BigintSchema, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput): BigintSchema<TOutput>;
declare function transform<TSchema extends BooleanSchema, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput): BooleanSchema<TOutput>;
declare function transform<TSchema extends BlobSchema, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput): BlobSchema<TOutput>;
declare function transform<TSchema extends DateSchema, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput): DateSchema<TOutput>;
declare function transform<TSchema extends EnumSchema<any>, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput): EnumSchema<TSchema['enum'], TOutput>;
declare function transform<TSchema extends InstanceSchema<any>, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput): InstanceSchema<TSchema['class'], TOutput>;
declare function transform<TSchema extends LiteralSchema<any>, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput): LiteralSchema<TSchema['literal'], TOutput>;
declare function transform<TSchema extends MapSchema<any, any>, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput): MapSchema<TSchema['map']['key'], TSchema['map']['value'], TOutput>;
declare function transform<TSchema extends NanSchema, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput): NanSchema<TOutput>;
declare function transform<TSchema extends NativeEnumSchema<any>, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput): NativeEnumSchema<TSchema['nativeEnum'], TOutput>;
declare function transform<TSchema extends NeverSchema>(schema: TSchema, action: (value: Output<TSchema>) => never): NeverSchema;
declare function transform<TSchema extends NonNullableSchema<any>, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput): NonNullableSchema<TSchema['wrapped'], TOutput>;
declare function transform<TSchema extends NonNullishSchema<any>, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput): NonNullishSchema<TSchema['wrapped'], TOutput>;
declare function transform<TSchema extends NonOptionalSchema<any>, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput): NonOptionalSchema<TSchema['wrapped'], TOutput>;
declare function transform<TSchema extends NullSchema, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput): NullSchema<TOutput>;
declare function transform<TSchema extends NullableSchema<any>, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput): NullableSchema<TSchema['wrapped'], TOutput>;
declare function transform<TSchema extends NullishSchema<any>, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput): NullishSchema<TSchema['wrapped'], TOutput>;
declare function transform<TSchema extends NumberSchema, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput): NumberSchema<TOutput>;
declare function transform<TSchema extends ObjectSchema<any>, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput): ObjectSchema<TSchema['object'], TOutput>;
declare function transform<TSchema extends OptionalSchema<any>, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput): OptionalSchema<TSchema['wrapped'], TOutput>;
declare function transform<TSchema extends RecordSchema<any, any>, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput): RecordSchema<TSchema['record']['key'], TSchema['record']['value'], TOutput>;
declare function transform<TSchema extends RecursiveSchema<any>, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput): RecursiveSchema<TSchema['getter'], TOutput>;
declare function transform<TSchema extends SetSchema<any>, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput): SetSchema<TSchema['set']['value'], TOutput>;
declare function transform<TSchema extends SpecialSchema<any>, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput): SpecialSchema<Input<TSchema>, TOutput>;
declare function transform<TSchema extends StringSchema, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput): StringSchema<TOutput>;
declare function transform<TSchema extends SymbolSchema, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput): SymbolSchema<TOutput>;
declare function transform<TSchema extends TupleSchema<any, any>, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput): TupleSchema<TSchema['tuple']['items'], TSchema['tuple']['rest'], TOutput>;
declare function transform<TSchema extends UndefinedSchema, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput): UndefinedSchema<TOutput>;
declare function transform<TSchema extends UnionSchema<any>, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput): UnionSchema<TSchema['union'], TOutput>;
declare function transform<TSchema extends UnknownSchema, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput): UnknownSchema<TOutput>;
declare function transform<TSchema extends VoidSchema, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput): VoidSchema<TOutput>;

declare function transformAsync<TSchema extends AnySchema | AnySchemaAsync, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput | Promise<TOutput>): AnySchemaAsync<TOutput>;
declare function transformAsync<TSchema extends ArraySchema<any> | ArraySchemaAsync<any>, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput | Promise<TOutput>): ArraySchemaAsync<TSchema['array']['item'], TOutput>;
declare function transformAsync<TSchema extends BigintSchema | BigintSchemaAsync, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput | Promise<TOutput>): BigintSchemaAsync<TOutput>;
declare function transformAsync<TSchema extends BooleanSchema | BooleanSchemaAsync, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput | Promise<TOutput>): BooleanSchemaAsync<TOutput>;
declare function transformAsync<TSchema extends BlobSchema | BlobSchemaAsync, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput | Promise<TOutput>): BlobSchemaAsync<TOutput>;
declare function transformAsync<TSchema extends DateSchema | DateSchemaAsync, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput | Promise<TOutput>): DateSchemaAsync<TOutput>;
declare function transformAsync<TSchema extends EnumSchema<any> | EnumSchemaAsync<any>, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput | Promise<TOutput>): EnumSchemaAsync<TSchema['enum'], TOutput>;
declare function transformAsync<TSchema extends InstanceSchema<any> | InstanceSchemaAsync<any>, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput | Promise<TOutput>): InstanceSchemaAsync<TSchema['class'], TOutput>;
declare function transformAsync<TSchema extends LiteralSchema<any> | LiteralSchemaAsync<any>, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput | Promise<TOutput>): LiteralSchemaAsync<TSchema['literal'], TOutput>;
declare function transformAsync<TSchema extends MapSchema<any, any> | MapSchemaAsync<any, any>, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput | Promise<TOutput>): MapSchemaAsync<TSchema['map']['key'], TSchema['map']['value'], TOutput>;
declare function transformAsync<TSchema extends NanSchema | NanSchemaAsync, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput | Promise<TOutput>): NanSchemaAsync<TOutput>;
declare function transformAsync<TSchema extends NativeEnumSchema<any> | NativeEnumSchemaAsync<any>, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput | Promise<TOutput>): NativeEnumSchemaAsync<TSchema['nativeEnum'], TOutput>;
declare function transformAsync<TSchema extends NeverSchema | NeverSchemaAsync>(schema: TSchema, action: (value: Output<TSchema>) => never): NeverSchemaAsync;
declare function transformAsync<TSchema extends NonNullableSchema<any> | NonNullableSchemaAsync<any>, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput | Promise<TOutput>): NonNullableSchemaAsync<TSchema['wrapped'], TOutput>;
declare function transformAsync<TSchema extends NonNullishSchema<any> | NonNullishSchemaAsync<any>, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput | Promise<TOutput>): NonNullishSchemaAsync<TSchema['wrapped'], TOutput>;
declare function transformAsync<TSchema extends NonOptionalSchema<any> | NonOptionalSchemaAsync<any>, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput | Promise<TOutput>): NonOptionalSchemaAsync<TSchema['wrapped'], TOutput>;
declare function transformAsync<TSchema extends NullSchema | NullSchemaAsync, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput | Promise<TOutput>): NullSchemaAsync<TOutput>;
declare function transformAsync<TSchema extends NullableSchema<any> | NullableSchemaAsync<any>, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput | Promise<TOutput>): NullableSchemaAsync<TSchema['wrapped'], TOutput>;
declare function transformAsync<TSchema extends NullishSchema<any> | NullishSchemaAsync<any>, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput | Promise<TOutput>): NullishSchemaAsync<TSchema['wrapped'], TOutput>;
declare function transformAsync<TSchema extends NumberSchema | NumberSchemaAsync, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput | Promise<TOutput>): NumberSchemaAsync<TOutput>;
declare function transformAsync<TSchema extends ObjectSchema<any> | ObjectSchemaAsync<any>, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput | Promise<TOutput>): ObjectSchemaAsync<TSchema['object'], TOutput>;
declare function transformAsync<TSchema extends OptionalSchema<any> | OptionalSchemaAsync<any>, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput | Promise<TOutput>): OptionalSchemaAsync<TSchema['wrapped'], TOutput>;
declare function transformAsync<TSchema extends RecordSchema<any, any> | RecordSchemaAsync<any, any>, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput | Promise<TOutput>): RecordSchemaAsync<TSchema['record']['key'], TSchema['record']['value'], TOutput>;
declare function transformAsync<TSchema extends RecursiveSchema<any> | RecursiveSchemaAsync<any>, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput | Promise<TOutput>): RecursiveSchemaAsync<TSchema['getter'], TOutput>;
declare function transformAsync<TSchema extends SetSchema<any> | SetSchemaAsync<any>, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput | Promise<TOutput>): SetSchemaAsync<TSchema['set']['value'], TOutput>;
declare function transformAsync<TSchema extends SpecialSchema<any> | SpecialSchemaAsync<any>, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput | Promise<TOutput>): SpecialSchemaAsync<Input<TSchema>, TOutput>;
declare function transformAsync<TSchema extends StringSchema | StringSchemaAsync, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput | Promise<TOutput>): StringSchemaAsync<TOutput>;
declare function transformAsync<TSchema extends SymbolSchema | SymbolSchemaAsync, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput | Promise<TOutput>): SymbolSchemaAsync<TOutput>;
declare function transformAsync<TSchema extends TupleSchema<any, any> | TupleSchemaAsync<any, any>, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput | Promise<TOutput>): TupleSchemaAsync<TSchema['tuple']['items'], TSchema['tuple']['rest'], TOutput>;
declare function transformAsync<TSchema extends UndefinedSchema | UndefinedSchemaAsync, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput | Promise<TOutput>): UndefinedSchemaAsync<TOutput>;
declare function transformAsync<TSchema extends UnionSchema<any> | UnionSchemaAsync<any>, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput | Promise<TOutput>): UnionSchemaAsync<TSchema['union'], TOutput>;
declare function transformAsync<TSchema extends UnknownSchema | UnknownSchemaAsync, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput | Promise<TOutput>): UnknownSchemaAsync<TOutput>;
declare function transformAsync<TSchema extends VoidSchema | VoidSchemaAsync, TOutput>(schema: TSchema, action: (value: Output<TSchema>) => TOutput | Promise<TOutput>): VoidSchemaAsync<TOutput>;

/**
 * Unwraps the wrapped schema.
 *
 * @param schema The schema to be unwrapped.
 *
 * @returns The unwrapped schema.
 */
declare function unwrap<TSchema extends OptionalSchema<any> | OptionalSchemaAsync<any> | NullableSchema<any> | NullableSchemaAsync<any> | NullishSchema<any> | NullishSchemaAsync<any> | NonOptionalSchema<any> | NonOptionalSchemaAsync<any> | NonNullableSchema<any> | NonNullableSchemaAsync<any> | NonNullishSchema<any> | NonNullishSchemaAsync<any>>(schema: TSchema): TSchema['wrapped'];

/**
 * Passes a default value to a schema in case of an undefined input.
 *
 * @deprecated Use `optional` instead.
 *
 * @param schema The affected schema.
 * @param value The default value.
 *
 * @returns The passed schema.
 */
declare function withDefault<TSchema extends BaseSchema | BaseSchemaAsync>(schema: TSchema, value: Input<TSchema> | (() => Input<TSchema>)): TSchema;
/**
 * See {@link withDefault}
 *
 * @deprecated Use `optional` instead.
 */
declare const useDefault: typeof withDefault;

/**
 * Creates a custom transformation function.
 *
 * @param action The transform action.
 *
 * @returns A transformation function.
 */
declare function toCustom<TInput>(action: (input: TInput) => TInput): (input: TInput) => PipeResult<TInput>;

/**
 * Creates a async custom transformation function.
 *
 * @param action The transform action.
 *
 * @returns A async transformation function.
 */
declare function toCustomAsync<TInput>(action: (input: TInput) => TInput | Promise<TInput>): (input: TInput) => Promise<PipeResult<TInput>>;

/**
 * Creates a transformation function that converts all the alphabetic
 * characters in a string to lowercase.
 *
 * @returns A transformation function.
 */
declare function toLowerCase(): (input: string) => PipeResult<string>;

/**
 * Creates a transformation function that sets a string, number or date to a
 * maximum value.
 *
 * @param requirement The maximum value.
 *
 * @returns A transformation function.
 */
declare function toMaxValue<TInput extends string | number | bigint | Date, TRequirement extends TInput>(requirement: TRequirement): (input: TInput) => PipeResult<TInput>;

/**
 * Creates a transformation function that sets a string, number or date to a
 * minimum value.
 *
 * @param requirement The minimum value.
 *
 * @returns A transformation function.
 */
declare function toMinValue<TInput extends string | number | bigint | Date, TRequirement extends TInput>(requirement: TRequirement): (input: TInput) => PipeResult<TInput>;

/**
 * Creates a transformation function that removes the leading and trailing
 * white space and line terminator characters from a string.
 *
 * @returns A transformation function.
 */
declare function toTrimmed(): (input: string) => PipeResult<string>;

/**
 * Creates a transformation function that removes the trailing white space and
 * line terminator characters from a string.
 *
 * @returns A transformation function.
 */
declare function toTrimmedEnd(): (input: string) => PipeResult<string>;

/**
 * Creates a transformation function that removes the leading white space and
 * line terminator characters from a string.
 *
 * @returns A transformation function.
 */
declare function toTrimmedStart(): (input: string) => PipeResult<string>;

/**
 * Creates a transformation function that converts all the alphabetic
 * characters in a string to uppercase.
 *
 * @returns A transformation function.
 */
declare function toUpperCase(): (input: string) => PipeResult<string>;

/**
 * Executes the validation and transformation pipe.
 *
 * @param input The input value.
 * @param pipe The pipe to be executed.
 * @param info The validation info.
 *
 * @returns The output value.
 */
declare function executePipe<TValue>(input: TValue, pipe: Pipe<TValue> | undefined, parseInfo: ParseInfo | undefined, reason: IssueReason): _ParseResult<TValue>;

/**
 * Executes the async validation and transformation pipe.
 *
 * @param input The input value.
 * @param pipe The pipe to be executed.
 * @param info The validation info.
 *
 * @returns The output value.
 */
declare function executePipeAsync<TValue>(input: TValue, pipe: PipeAsync<TValue> | undefined, parseInfo: ParseInfo | undefined, reason: IssueReason): Promise<_ParseResult<TValue>>;

/**
 * Returns error and pipe from dynamic arguments.
 *
 * @param arg1 First argument.
 * @param arg2 Second argument.
 *
 * @returns The default arguments.
 */
declare function getDefaultArgs<TPipe extends Pipe<any> | PipeAsync<any>>(arg1: ErrorMessage | TPipe | undefined, arg2: TPipe | undefined): [ErrorMessage | undefined, TPipe | undefined];

/**
 * Returns the result object with issues.
 *
 * @param issues The issues.
 *
 * @returns The result object.
 */
declare function getIssues<TIssues>(issues: TIssues): {
    issues: TIssues;
};

/**
 * Returns the result object with an output.
 *
 * @param output The output value.
 *
 * @returns The result object.
 */
declare function getOutput<TOutput>(output: TOutput): {
    output: TOutput;
};

/**
 * Returns the pipeline result object with issues.
 *
 * @param validation The validation name.
 * @param message The error message.
 * @param input The input value.
 *
 * @returns The pipeline result object.
 */
declare function getPipeIssues(validation: string, error: ErrorMessage, input: unknown): {
    issues: Pick<Issue, 'validation' | 'message' | 'input' | 'path'>[];
};

/**
 * Returns the schema result object with issues.
 *
 * @param info The parse info.
 * @param reason The issue reason.
 * @param validation The validation name.
 * @param message The error message.
 * @param input The input value.
 * @param issues The sub issues.
 *
 * @returns The schema result object.
 */
declare function getSchemaIssues(info: ParseInfo | undefined, reason: IssueReason, validation: string, error: ErrorMessage, input: unknown, issues?: Issues): {
    issues: Issues;
};

/**
 * Checks whether a string with numbers corresponds to the luhn algorithm.
 *
 * @param input The input to be checked.
 *
 * @returns Whether input is valid.
 */
declare function isLuhnAlgo(input: string): boolean;

/**
 * Creates a validation functions that validates the byte length of a string.
 *
 * @param requirement The byte length.
 * @param error The error message.
 *
 * @returns A validation function.
 */
declare function bytes<TInput extends string>(requirement: number, error?: ErrorMessage): (input: TInput) => PipeResult<TInput>;

/**
 * Creates a validation functions that validates a [cuid2](https://github.com/paralleldrive/cuid2#cuid2).
 *
 * @param error The error message.
 *
 * @returns A validation function.
 */
declare function cuid2<TInput extends string>(error?: ErrorMessage): (input: TInput) => PipeResult<TInput>;

/**
 * Creates a custom validation function.
 *
 * @param requirement The validation function.
 * @param error The error message.
 *
 * @returns A validation function.
 */
declare function custom<TInput>(requirement: (input: TInput) => boolean, error?: ErrorMessage): (input: TInput) => PipeResult<TInput>;

/**
 * Creates a async custom validation function.
 *
 * @param requirement The async validation function.
 * @param error The error message.
 *
 * @returns A async validation function.
 */
declare function customAsync<TInput>(requirement: (input: TInput) => Promise<boolean>, error?: ErrorMessage): (input: TInput) => Promise<PipeResult<TInput>>;

/**
 * Creates a validation functions that validates a email.
 *
 * Hint: The regex used is not perfect, but should work for most emails.
 *
 * @param error The error message.
 *
 * @returns A validation function.
 */
declare function email<TInput extends string>(error?: ErrorMessage): (input: TInput) => PipeResult<TInput>;

/**
 * Creates a validation functions that validates a emoji.
 *
 * @param error The error message.
 *
 * @returns A validation function.
 */
declare function emoji<TInput extends string>(error?: ErrorMessage): (input: TInput) => PipeResult<TInput>;

/**
 * Creates a validation functions that validates the end of a string.
 *
 * @param requirement The end string.
 * @param error The error message.
 *
 * @returns A validation function.
 */
declare function endsWith<TInput extends string>(requirement: string, error?: ErrorMessage): (input: TInput) => PipeResult<TInput>;

/**
 * Creates a validation function that checks the value for equality.
 *
 * @param requirement The required value.
 * @param error The error message.
 *
 * @returns A validation function.
 */
declare function equal<TInput extends string | number | bigint | boolean, TRequirement extends TInput>(requirement: TRequirement, error?: ErrorMessage): (input: TInput) => PipeResult<TInput>;

declare function excludes<TInput extends string>(requirement: string, error?: ErrorMessage): (input: TInput) => PipeResult<TInput>;
declare function excludes<TInput extends TItem[], TItem>(requirement: TItem, error?: ErrorMessage): (input: TInput) => PipeResult<TInput>;

/**
 * Creates a validation function that validates whether a number is finite.
 *
 * @param error The error message.
 *
 * @returns A validation function.
 */
declare function finite<TInput extends number>(error?: ErrorMessage): (input: TInput) => PipeResult<TInput>;

/**
 * Creates a validation functions that validates a IMEI.
 *
 * Format: AA-BBBBBB-CCCCCC-D
 *
 * @param error The error message.
 *
 * @returns A validation function.
 */
declare function imei<TInput extends string>(error?: ErrorMessage): (input: TInput) => PipeResult<TInput>;

declare function includes<TInput extends string>(requirement: string, error?: ErrorMessage): (input: TInput) => PipeResult<TInput>;
declare function includes<TInput extends TItem[], TItem>(requirement: TItem, error?: ErrorMessage): (input: TInput) => PipeResult<TInput>;

/**
 * Creates a validation function that validates whether a number is an integer.
 *
 * @param error The error message.
 *
 * @returns A validation function.
 */
declare function integer<TInput extends number>(error?: ErrorMessage): (input: TInput) => PipeResult<TInput>;

/**
 * Creates a validation functions that validates a IP v4 or v6 address.
 *
 * @param error The error message.
 *
 * @returns A validation function.
 */
declare function ip<TInput extends string>(error?: ErrorMessage): (input: TInput) => PipeResult<TInput>;

/**
 * Creates a validation functions that validates a IP v4 address.
 *
 * @param error The error message.
 *
 * @returns A validation function.
 */
declare function ipv4<TInput extends string>(error?: ErrorMessage): (input: TInput) => PipeResult<TInput>;

/**
 * Creates a validation functions that validates a IP v6 address.
 *
 * @param error The error message.
 *
 * @returns A validation function.
 */
declare function ipv6<TInput extends string>(error?: ErrorMessage): (input: TInput) => PipeResult<TInput>;

/**
 * Creates a validation functions that validates a date.
 *
 * Format: yyyy-mm-dd
 *
 * Hint: The regex used cannot validate the maximum number of days based on
 * year and month. For example, "2023-06-31" is valid although June has only
 * 30 days.
 *
 * @param error The error message.
 *
 * @returns A validation function.
 */
declare function isoDate<TInput extends string>(error?: ErrorMessage): (input: TInput) => PipeResult<TInput>;

/**
 * Creates a validation functions that validates a datetime.
 *
 * Format: yyyy-mm-ddThh:mm
 *
 * Hint: The regex used cannot validate the maximum number of days based on
 * year and month. For example, "2023-06-31T00:00" is valid although June has only
 * 30 days.
 *
 * @param error The error message.
 *
 * @returns A validation function.
 */
declare function isoDateTime<TInput extends string>(error?: ErrorMessage): (input: TInput) => PipeResult<TInput>;

/**
 * Creates a validation functions that validates a time.
 *
 * Format: hh:mm
 *
 * @param error The error message.
 *
 * @returns A validation function.
 */
declare function isoTime<TInput extends string>(error?: ErrorMessage): (input: TInput) => PipeResult<TInput>;

/**
 * Creates a validation functions that validates a time with seconds.
 *
 * Format: hh:mm:ss
 *
 * @param error The error message.
 *
 * @returns A validation function.
 */
declare function isoTimeSecond<TInput extends string>(error?: ErrorMessage): (input: TInput) => PipeResult<TInput>;

/**
 * Creates a validation functions that validates a timestamp.
 *
 * Format: yyyy-mm-ddThh:mm:ss.sssZ
 *
 * Hint: The regex used cannot validate the maximum number of days based on
 * year and month. For example, "2023-06-31T00:00:00.000Z" is valid although
 * June has only 30 days.
 *
 * @param error The error message.
 *
 * @returns A validation function.
 */
declare function isoTimestamp<TInput extends string>(error?: ErrorMessage): (input: TInput) => PipeResult<TInput>;

/**
 * Creates a validation functions that validates a week.
 *
 * Format: yyyy-Www
 *
 * Hint: The regex used cannot validate the maximum number of weeks based on
 * the year. For example, "2021W53" is valid even though the year 2021 has only
 * 52 weeks.
 *
 * @param error The error message.
 *
 * @returns A validation function.
 */
declare function isoWeek<TInput extends string>(error?: ErrorMessage): (input: TInput) => PipeResult<TInput>;

/**
 * Creates a validation functions that validates the length of a string or array.
 *
 * @param requirement The length.
 * @param error The error message.
 *
 * @returns A validation function.
 */
declare function length<TInput extends string | any[]>(requirement: number, error?: ErrorMessage): (input: TInput) => PipeResult<TInput>;

/**
 * Creates a validation functions that validates the byte length of a string.
 *
 * @param requirement The maximum length in byte.
 * @param error The error message.
 *
 * @returns A validation function.
 */
declare function maxBytes<TInput extends string>(requirement: number, error?: ErrorMessage): (input: TInput) => PipeResult<TInput>;

/**
 * Creates a validation functions that validates the length of a string or array.
 *
 * @param requirement The maximum length.
 * @param error The error message.
 *
 * @returns A validation function.
 */
declare function maxLength<TInput extends string | any[]>(requirement: number, error?: ErrorMessage): (input: TInput) => PipeResult<TInput>;

/**
 * Creates a validation functions that validates the size of a map, set or blob.
 *
 * @param requirement The maximum size.
 * @param error The error message.
 *
 * @returns A validation function.
 */
declare function maxSize<TInput extends Map<any, any> | Set<any> | Blob>(requirement: number, error?: ErrorMessage): (input: TInput) => PipeResult<TInput>;

/**
 * Creates a validation functions that validates the value of a string, number or date.
 *
 * @param requirement The maximum value.
 * @param error The error message.
 *
 * @returns A validation function.
 */
declare function maxValue<TInput extends string | number | bigint | Date, TRequirement extends TInput>(requirement: TRequirement, error?: ErrorMessage): (input: TInput) => PipeResult<TInput>;
/**
 * See {@link maxValue}
 *
 * @deprecated Function has been renamed to `maxValue`.
 */
declare const maxRange: typeof maxValue;

/**
 * Creates a validation functions that validates the byte length of a string.
 *
 * @param requirement The minimum length in byte.
 * @param error The error message.
 *
 * @returns A validation function.
 */
declare function minBytes<TInput extends string>(requirement: number, error?: ErrorMessage): (input: TInput) => PipeResult<TInput>;

/**
 * Creates a validation functions that validates the MIME type of a file.
 *
 * @param requirement The MIME types.
 * @param error The error message.
 *
 * @returns A validation function.
 */
declare function mimeType<TInput extends Blob>(requirement: `${string}/${string}`[], error?: ErrorMessage): (input: TInput) => PipeResult<TInput>;

/**
 * Creates a validation functions that validates the length of a string or array.
 *
 * @param requirement The minimum length.
 * @param error The error message.
 *
 * @returns A validation function.
 */
declare function minLength<TInput extends string | any[]>(requirement: number, error?: ErrorMessage): (input: TInput) => PipeResult<TInput>;

/**
 * Creates a validation functions that validates the size of a map, set or blob.
 *
 * @param requirement The minimum size.
 * @param error The error message.
 *
 * @returns A validation function.
 */
declare function minSize<TInput extends Map<any, any> | Set<any> | Blob>(requirement: number, error?: ErrorMessage): (input: TInput) => PipeResult<TInput>;

/**
 * Creates a validation functions that validates the value of a string, number or date.
 *
 * @param requirement The minimum value.
 * @param error The error message.
 *
 * @returns A validation function.
 */
declare function minValue<TInput extends string | number | bigint | Date, TRequirement extends TInput>(requirement: TRequirement, error?: ErrorMessage): (input: TInput) => PipeResult<TInput>;
/**
 * See {@link minValue}
 *
 * @deprecated Function has been renamed to `minValue`.
 */
declare const minRange: typeof minValue;

/**
 * Creates a validation function that validates whether a number is a multiple.
 *
 * @param requirement The divisor.
 * @param error The error message.
 *
 * @returns A validation function.
 */
declare function multipleOf<TInput extends number>(requirement: number, error?: ErrorMessage): (input: TInput) => PipeResult<TInput>;

/**
 * Creates a validation functions that validates a string with a regex.
 *
 * @param requirement The regex pattern.
 * @param error The error message.
 *
 * @returns A validation function.
 */
declare function regex<TInput extends string>(requirement: RegExp, error?: ErrorMessage): (input: TInput) => PipeResult<TInput>;

/**
 * Creates a validation function that validates whether a number is a safe integer.
 *
 * @param error The error message.
 *
 * @returns A validation function.
 */
declare function safeInteger<TInput extends number>(error?: ErrorMessage): (input: TInput) => PipeResult<TInput>;

/**
 * Creates a validation functions that validates the size of a map, set or blob.
 *
 * @param requirement The size.
 * @param error The error message.
 *
 * @returns A validation function.
 */
declare function size<TInput extends Map<any, any> | Set<any> | Blob>(requirement: number, error?: ErrorMessage): (input: TInput) => PipeResult<TInput>;

/**
 * Creates a validation functions that validates the start of a string.
 *
 * @param requirement The start string.
 * @param error The error message.
 *
 * @returns A validation function.
 */
declare function startsWith<TInput extends string>(requirement: string, error?: ErrorMessage): (input: TInput) => PipeResult<TInput>;

/**
 * Creates a validation functions that validates a [ULID](https://github.com/ulid/spec).
 *
 * @param error The error message.
 *
 * @returns A validation function.
 */
declare function ulid<TInput extends string>(error?: ErrorMessage): (input: TInput) => PipeResult<TInput>;

/**
 * Creates a validation functions that validates a URL.
 *
 * Hint: The value is passed to the URL constructor to check if it is valid.
 * This check is not perfect. For example, values like "abc:1234" are accepted.
 *
 * @param error The error message.
 *
 * @returns A validation function.
 */
declare function url<TInput extends string>(error?: ErrorMessage): (input: TInput) => PipeResult<TInput>;

/**
 * Creates a validation functions that validates a UUID.
 *
 * @param error The error message.
 *
 * @returns A validation function.
 */
declare function uuid<TInput extends string>(error?: ErrorMessage): (input: TInput) => PipeResult<TInput>;

/**
 * Creates a validation functions that validates the value of a string or number.
 *
 * @param requirement The value.
 * @param error The error message.
 *
 * @returns A validation function.
 */
declare function value<TInput extends string | number | bigint, TRequirement extends TInput>(requirement: TRequirement, error?: ErrorMessage): (input: TInput) => PipeResult<TInput>;

export { AnySchema, AnySchemaAsync, ArrayPathItem, ArraySchema, ArraySchemaAsync, BaseSchema, BaseSchemaAsync, BigintSchema, BigintSchemaAsync, BlobSchema, BlobSchemaAsync, BooleanSchema, BooleanSchemaAsync, Brand, BrandSymbol, Class, DateSchema, DateSchemaAsync, Enum, EnumSchema, EnumSchemaAsync, ErrorMessage, FlatErrors, Input, InstanceSchema, InstanceSchemaAsync, Issue, IssueOrigin, IssueReason, Issues, Literal, LiteralSchema, LiteralSchemaAsync, MapInput, MapOutput, MapPathItem, MapSchema, MapSchemaAsync, NanSchema, NanSchemaAsync, NativeEnum, NativeEnumSchema, NativeEnumSchemaAsync, NeverSchema, NeverSchemaAsync, NonNullable$1 as NonNullable, NonNullableSchema, NonNullableSchemaAsync, NonNullish, NonNullishSchema, NonNullishSchemaAsync, NonOptional, NonOptionalSchema, NonOptionalSchemaAsync, NullSchema, NullSchemaAsync, NullableSchema, NullableSchemaAsync, NullishSchema, NullishSchemaAsync, NumberSchema, NumberSchemaAsync, ObjectInput, ObjectOutput, ObjectPathItem, ObjectSchema, ObjectSchemaAsync, ObjectShape, ObjectShapeAsync, OptionalSchema, OptionalSchemaAsync, Output, ParseInfo, PathItem, Pipe, PipeAsync, PipeInfo, PipeResult, RecordInput, RecordKey, RecordKeyAsync, RecordOutput, RecordPathItem, RecordSchema, RecordSchemaAsync, RecursiveSchema, RecursiveSchemaAsync, ResolveObject, SafeParseResult, SetInput, SetOutput, SetPathItem, SetSchema, SetSchemaAsync, SpecialSchema, SpecialSchemaAsync, StringSchema, StringSchemaAsync, SymbolSchema, SymbolSchemaAsync, TupleInput, TupleOutput, TuplePathItem, TupleSchema, TupleSchemaAsync, TupleShape, TupleShapeAsync, UndefinedSchema, UndefinedSchemaAsync, UnionOptions, UnionOptionsAsync, UnionSchema, UnionSchemaAsync, UnknownSchema, UnknownSchemaAsync, ValiError, VoidSchema, VoidSchemaAsync, _ParseResult, any, anyAsync, array, arrayAsync, bigint, bigintAsync, blob, blobAsync, boolean, booleanAsync, brand, bytes, coerce, coerceAsync, cuid2, custom, customAsync, date, dateAsync, email, emoji, endsWith, enumType, enumTypeAsync, equal, excludes, executePipe, executePipeAsync, fallback, fallbackAsync, finite, flatten, getDefault, getDefaultArgs, getIssues, getOutput, getPipeIssues, getSchemaIssues, imei, includes, instance, instanceAsync, integer, ip, ipv4, ipv6, is, isLuhnAlgo, isoDate, isoDateTime, isoTime, isoTimeSecond, isoTimestamp, isoWeek, keyof, length, literal, literalAsync, map, mapAsync, maxBytes, maxLength, maxRange, maxSize, maxValue, merge, mergeAsync, mimeType, minBytes, minLength, minRange, minSize, minValue, multipleOf, nan, nanAsync, nativeEnum, nativeEnumAsync, never, neverAsync, nonNullable, nonNullableAsync, nonNullish, nonNullishAsync, nonOptional, nonOptionalAsync, nullType, nullTypeAsync, nullable, nullableAsync, nullish, nullishAsync, number, numberAsync, object, objectAsync, omit, omitAsync, optional, optionalAsync, parse, parseAsync, partial, partialAsync, passthrough, passthroughAsync, pick, pickAsync, record, recordAsync, recursive, recursiveAsync, regex, required, requiredAsync, safeInteger, safeParse, safeParseAsync, set, setAsync, size, special, specialAsync, startsWith, strict, strictAsync, string, stringAsync, strip, stripAsync, symbol, symbolAsync, toCustom, toCustomAsync, toLowerCase, toMaxValue, toMinValue, toTrimmed, toTrimmedEnd, toTrimmedStart, toUpperCase, transform, transformAsync, tuple, tupleAsync, ulid, undefinedType, undefinedTypeAsync, union, unionAsync, unknown, unknownAsync, unwrap, url, useDefault, uuid, value, voidType, voidTypeAsync, withDefault };
